; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *                        Sprite drawing code                          *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************

.386p 
.model flat 
.code 
include Gb.Inc
.data
dNormal         equ     0                       ; normal (nonpatch) mode
dSingleStep     equ     1                       ; single step mode
dRunBreak       equ     2                       ; run to breakpoint mode
PanelPosition   equ     8*320+8                 ; panel position on screen

align dword
JumpVector      dd      GoDOS                   ; patch value
Break1          dw      0FFFFh                  ; breakpoints
Break2          dw      0FFFFh
Break3          dw      0FFFFh
Break4          dw      0FFFFh
DataAddr        dw      00000h                  ; data address
CodeAddr        dw      00100h                  ; code address
StepOverBreak   dw      0FFFFh                  ; stepover break
old             db      0
DebugMode       db      dNormal                 ; Debugging mode
.code

TtlCol          equ     11
DatCol          equ     14
CodeKey         equ     1Dh

; ***********************************************************************
; *                                                                     *
; *          Patch in the Debugger if required.                         *
; *                                                                     *
; ***********************************************************************

DebugPatchIn Proc
        push    eax

        mov     eax,[DebugKey]                  ; if debug key pressed
        mov     al,[byte ptr KeyTable+eax]
        test    al,128
        jz      NoDebug
        mov     [DebugMode],dSingleStep         ; go into single step mode
NoDebug:cmp     [DebugMode],dNormal             ; if not in normal mode
        je      NoPatchEdi
        mov     [JumpVector],edi                ; will go to debug code at end
        mov     edi,offset DebugEntryPoint      ; e.g. here
NoPatchEdi:
        pop     eax
        ret
EndP DebugPatchIn

; ***********************************************************************
; *                                                                     *
; *     When the debugger is patched in you jump here rather than       *
; *     back to the start of the execute loop                           *
; *                                                                     *
; ***********************************************************************

DebugEntryPoint Proc
        push    eax

;       mov     al,[byte ptr Memory+0FFC0h]
;       cmp     al,[Old]
;       je      x1
;       mov     [Old],al
;       call    DebugByte
x1:

        cmp     [DebugMode],dSingleStep         ; single stepping ?
        je      GoSingleStep
        mov     ax,[ZRegPC]                     ; test breakpoints
        cmp     ax,[Break1]
        je      GoSingleStep
        cmp     ax,[Break2]
        je      GoSingleStep
        cmp     ax,[Break3]
        je      GoSingleStep
        cmp     ax,[Break4]
        je      GoSingleStep
        cmp     ax,[StepOverBreak]
        je      GoSingleStep
DEPExit:
        pop     eax
        jmp     [JumpVector]

GoSingleStep:
        push    eax ebx ecx edx ebp esi edi     ; save the registers
        mov     ax,0FFFFh                       ; clear the stepover break
        mov     [StepOverBreak],ax
        mov     [DebugMode],dSingleStep         ; go to debug mode
        call    UpdateCPUFlagByte               ; update F from AH
        call    DebugPanel                      ; display the panel
        call    ClearScreen                     ; redraw the screen
        call    [PaintVector]
        pop     edi esi ebp edx ecx ebx eax     ; restore the registers
        jmp     DEPExit
EndP DebugEntryPoint

; ***********************************************************************
; *                                                                     *
; *                     Display the debugging panel                     *
; *                                                                     *
; ***********************************************************************

DebugPanel Proc
        mov     ax,[ZRegPC]                     ; reset the code address
        mov     [CodeAddr],ax
        call    ClearPanel              ; Draw everything
        call    RefreshPanel
        call    RefreshData
        call    RefreshCode

DPanel1:call    CurrentKey              ; wait for key release
        cmp     al,255
        jne     DPanel1
DPanel2:call    CurrentKey              ; wait for key press
        cmp     al,255
        je      DPanel2
        push    eax
DPanel3:call    CurrentKey
        cmp     al,255
        jne     DPanel3
        pop     eax
        cmp     al,25                   ; is it step over ?
        je      DPanelStepOver
        cmp     al,24                   ; if it's single step then exit
        je      DPanelExit
        cmp     al,20                   ; if it's escape then exit,mode=0
        je      DPanelRunNormal
        jb      DPanelMove              ; < 20 it's a movement key
        cmp     al,23                   ; if it's run to break
        je      DPanelRunCheck
        call    DebugMisc
        jmp     DPanel1

DPanelMove:
        test    [byte ptr KeyTable+CodeKey],128
        jnz     DPMData
        mov     bx,[CodeAddr]           ; move the code
        call    MoveAddress
        mov     [CodeAddr],bx
        call    RefreshCode
        jmp     DPanel1
DPMData:mov     bx,[DataAddr]           ; move the data
        call    MoveAddress
        mov     [DataAddr],bx
        call    RefreshData
        jmp     DPanel1

DPanelStepOver:
        mov     bx,[ZRegPC]
        call    AtomTranslate
        inc     bx                      ; step one on
        mov     al,[esi]                ; the next instruction
        cmp     al,0C0h                 ; 00-BF are always step
        jb      DPanelExit
        and     eax,3Fh                 ; get only the relevant bits
        mov     al,[byte ptr StepTab+eax]
        cmp     al,0
        jz      DPanelExit
        mov     bx,[ZRegPC]
        add     bx,ax
DPS1:   mov     [StepOverBreak],bx      ; and set the 'breakpoint'

DPanelRunCheck:                         ; go into run but check mode
        mov     [DebugMode],dRunBreak
        jmp     DPanelExit

StepTab:db      0,0,0,0,3,0,0,1,0,0,0,0,3,3,0,1
        db      0,0,0,0,3,0,0,1,0,0,0,0,3,0,0,1
        db      0,0,0,0,3,0,0,1,0,0,0,0,3,0,0,1
        db      0,0,0,0,3,0,0,1,0,0,0,0,3,0,0,1

DPanelRunNormal:
        mov     [DebugMode],dNormal     ; remove debugging...
DPanelExit:
        ret
EndP DebugPanel

DebugMisc Proc
        cmp     al,21
        je      KeyHome
        cmp     al,22
        je      KeyEnd
        cmp     al,26
        jae     KeyBreak
        ret
KeyHome:mov     ax,[ZRegPC]             ; Key 21 - CodeAddr = PC
        mov     [CodeAddr],ax
        call    RefreshCode
        ret
KeyEnd: mov     ax,0FF00h               ; Key 22 - DataAddr = $FF00
        mov     [DataAddr],ax
        call    RefreshData
        ret
KeyBreak:
        sub     al,26                   ; 26-29 get/clr the breakpoint
        and     eax,3
        mov     bx,[Break1+eax*2]
        mov     cx,[CodeAddr]
        cmp     bx,cx
        jne     KBNoReset
        mov     cx,0FFFFh
KBNoReset:
        mov     [Break1+eax*2],cx
        call    RefreshPanel
        ret



EndP DebugMisc

MoveAddress Proc
        cmp     al,16                   ; >= 16 = move in direction
        jae     MADirection
        shl     bx,4                    ; shift into lsb of address
        and     al,15
        or      bl,al
        ret
MADirection:                            ; arrow keys,page up & down
        and     eax,03
        add     bx,[word ptr MADir+eax*2]
        ret
MADir:  dw      -8,8,-64,64
EndP MoveAddress
     
; ***********************************************************************
; *                                                                     *
; *                          Get a keystroke                            *
; *                                                                     *
; ***********************************************************************

CurrentKey      Proc
        push    ebx
        xor     eax,eax
        xor     ebx,ebx
CKey1:  mov     bl,[byte ptr DebugKeyTable+eax]
        cmp     bl,0
        jz      CKeyNone
        test    [byte ptr KeyTable+ebx],128     ; key pressed ?
        jnz     CKExit
        inc     eax
        jmp     CKey1
CKeyNone:mov    eax,-1                          ; no key , return -1
CKExit: pop     ebx
        ret
EndP CurrentKey

.data
DebugKeyTable:
        db      0bh,2,3,4,5,6,7,8,9             ; keys 0-F for data entry
        db      0ah,1eh,30h,2eh,20h,12h,21h     ; (key 0..15)
        db      48h,50h,49h,51h                 ; up/down/pup/pdown (16..19)
        db      44h                             ; F10 (20) run normally
        db      47h                             ; home (21) CodeAddr=PC
        db      4Fh                             ; end (22) DataAddr=$FF00
        db      3Fh                             ; F5 (23) run to break
        db      41h                             ; F7 (24) single step
        db      42h                             ; F8 (25) step through
        db      3bh,3ch,3dh,3eh                 ; F9 (26) breakpoints
        db      0
.code

; ***********************************************************************
; *                                                                     *
; *                     Refresh the code area                           *
; *                                                                     *
; ***********************************************************************

RefreshCode Proc
        push    eax ebx ecx
        mov     ebx,PanelPosition+112
        mov     ax,[CodeAddr]
        mov     cl,13
RCLoop: call    RCPart1
        add     ebx,320*8
        dec     cl
        jnz     RCLoop
        pop     ecx ebx eax
        ret

RCPart1:push    ebx ecx edx esi ebp
        mov     dx,ax
        push    ebx                             ; clear rest of the line
        add     ebx,[VideoRAM]
        xor     eax,eax
        mov     cl,43
RCLoop3:mov     [ebx],eax
        mov     [ebx+320],eax
        mov     [ebx+640],eax
        mov     [ebx+960],eax
        mov     [ebx+1280],eax
        mov     [ebx+1600],eax
        mov     [ebx+1920],eax
        mov     [ebx+2240],eax
        add     ebx,4
        dec     cl
        jnz     RCLoop3
        pop     ebx

        mov     al,12                           ; display the marker ?
        call    TxtColour
        mov     al,'>'
        cmp     dx,[zregpc]
        je      RC1a
        mov     al,' '
Rc1a:   call    TxtCharacter

        mov     al,TtlCol                       ; display the address
        call    TxtColour
        mov     ax,dx
        call    TxtWord
        push    ebx edx                         ; ESI ^ Data DX ^ Address
        mov     bx,dx
        call    AtomTranslate
        pop     edx ebx

        mov     al,DatCol                       ; set colour
        call    TxtColour
        mov     al,32
        call    TxtCharacter

        xor     eax,eax                         ; AL = Byte
        mov     al,[esi]                        ; read it in
        inc     esi                             ; advance pointer & address
        inc     dx
        cmp     al,0CBh                         ; is it extend CB ?
        je      ExtendCB

        shl     eax,4                           ; multiply by 16
        add     eax,offset DisAsm+1             ; now points to the text
        mov     ebp,eax                         ; put the pointer into EBP
RCLoop2:mov     al,[ebp]
        inc     ebp
        cmp     al,'!'                          ; ! is exit
        je      RCExit
        cmp     al,'#'                          ; # is word
        je      RCWord
        cmp     al,'*'                          ; * is byte
        je      RCByte
        cmp     al,'@'                          ; @ is sign
        je      RCSign
        call    TxtCharacter
        jmp     RCLoop2

RCSign: mov     al,[esi]                        ; signed operand
        inc     esi
        inc     dx
        mov     ah,'+'
        test    al,128
        jz      RCSign2
        mov     ah,'-'
        xor     al,255
        inc     al
RCSign2:push    eax
        mov     al,ah
        call    TxtCharacter
        pop     eax
        call    TxtByte
        jmp     RCLoop2
RCByte: mov     al,[esi]                        ; byte operand
        inc     esi
        inc     dx
        call    TxtByte
        jmp     RCLoop2

RCWord: mov     ax,[esi]                ; word operand
        add     esi,2
        add     dx,2
        call    TxtWord
        jmp     RCLoop2

ExtendCB:
        mov     al,[esi]
        inc     esi
        inc     dx
        call    DisasmCB
        jmp     RCExit

RCExit: mov     ax,dx
        pop     ebp esi edx ecx ebx
        ret

EndP    RefreshCode

DisasmCB Proc
        push    eax ebx ecx edx
        and     eax,0FFh                ; make it 32 bit
        push    eax                     ; do the operation bit
        shr     eax,3
        shl     eax,2
        mov     edx,offset DCB1
        add     edx,eax
        call    TxtString
        mov     al,' '                  ; do space
        call    TxtCharacter
        pop     eax
        cmp     al,40h                  ; is it a bit op ?
        jb      NotBitOp
        push    eax
        shr     eax,3                   ; do the bit/set/res bit
        and     al,7
        add     al,48
        call    TxtCharacter
        mov     al,','                  ; do comma
        call    TxtCharacter
        pop     eax
NotBitOp:
        and     al,7                    ; finally the register
        mov     al,[byte ptr DCB2+eax]
        cmp     al,'*'
        je      DCBHL
        call    TxtCharacter
        jmp     DCBExit
DCBHL:  mov     edx,offset DCB3
        call    TxtString
DCBExit:pop     edx ecx ebx eax
        ret

.data
DCB1:   db      'rlc',0,'rrc',0,'rl ',0,'rr ',0
        db      'sla',0,'sra',0,'swp',0,'srl',0
        db      'bit',0,'bit',0,'bit',0,'bit',0
        db      'bit',0,'bit',0,'bit',0,'bit',0
        db      'res',0,'res',0,'res',0,'res',0
        db      'res',0,'res',0,'res',0,'res',0
        db      'set',0,'set',0,'set',0,'set',0
        db      'set',0,'set',0,'set',0,'set',0
DCB2:   db      'bcdehl*a'
DCB3:   db      '(hl)',0
.code
EndP DisasmCB

.data                                   ; disassembly lookup table
DisAsm:
        include disasm.inc
        db      0
.code

; ***********************************************************************
; *                                                                     *
; *                     Refresh the data area                           *
; *                                                                     *
; ***********************************************************************

RefreshData Proc
        push    eax ebx ecx
        mov     ebx,PanelPosition+104+14*320*8
        mov     ax,[DataAddr]
        mov     cl,8
RDLoop: call    RDPart1
        add     eax,8
        add     ebx,320*8
        dec     cl
        jnz     RDLoop
        pop     ecx ebx eax
        ret

RDPart1:push    eax ebx ecx edx esi
        mov     edx,eax
        mov     al,TtlCol
        call    TxtColour
        mov     ax,dx
        call    TxtWord
        add     ebx,4
        push    ebx
        mov     bx,dx
        call    AtomTranslate
        pop     ebx
        mov     ah,8
        mov     al,DatCol
        call    TxtColour
RDPart2:mov     al,[esi]
        call    TxtByte
        inc     esi
        add     ebx,3
        dec     ah
        jnz     RDPart2
        pop     esi edx ecx ebx eax
        ret
EndP RefreshData

; ***********************************************************************
; *                                                                     *
; *                   Clear the debugging panel                         *
; *                                                                     *
; ***********************************************************************

ClearPanel Proc
        push    eax ebx ecx
        mov     ebx,PanelPosition-4-4*320
        add     ebx,[VideoRAM]
        mov     cl,184
CLP1:   xor     eax,eax
        cmp     cl,184
        je      CLP3
        cmp     cl,1
        jne     CLP4
CLP3:   mov     eax,0f0f0f0fh
CLP4:   mov     ch,15
        mov     [ebx-1],ch
        mov     ch,296/4
        push    ebx
CLP2:   mov     [ebx],eax
        add     ebx,4
        dec     ch
        jnz     CLP2
        mov     ch,15
        mov     [ebx],ch
        pop     ebx
        mov     [ebx+13*8],ch
        add     ebx,320
        dec     cl
        jnz     CLP1
        mov     ebx,PanelPosition+13*8-4+14*320*8-4*320
        add     ebx,[VideoRAM]
        mov     al,15
        mov     ah,192
CLP5:   mov     [ebx],al
        inc     ebx
        dec     ah
        jnz     CLP5
        pop     ecx ebx eax
        ret
EndP ClearPanel

; ***********************************************************************
; *                                                                     *
; *                     Refresh the variables                           *
; *                                                                     *
; ***********************************************************************

RefreshPanel Proc
        push    eax ebx ecx esi
        mov     esi,offset PanelInfo    ; start of information
        mov     ebx,PanelPosition
RPLoop: mov     ecx,[esi]               ; read the address to read
        cmp     ecx,0                   ; end of table ?
        jz      RPExit
        add     esi,4                   ; skip the rest of it
        push    ebx                     ; save the drawing address
        mov     al,TtlCol               ; set the colour
        call    TxtColour
RPName: mov     al,[esi]                ; read the first character
        cmp     al,32                   ; if < 32 reached the end
        jb      EndOfName
        call    TxtCharacter            ; print the character
        inc     esi
        jmp     RPName                  ; do the next one.
EndOfName:
        pop     ebx                     ; restore & resave the screen pointer
        push    ebx
        add     ebx,8*8
        mov     al,DatCol               ; display the data
        call    TxtColour
        mov     al,[esi]
        cmp     al,1
        je      RPSingle
        cmp     al,3
        jne     RPWord
        mov     ax,[ecx]                ; if breakpoint set display it
        cmp     ax,0FFFFh
        jne     RPWord
        mov     al,' '
        call    TxtCharacter
        call    TxtCharacter
        call    TxtCharacter
        call    TxtCharacter
        jmp     RPNext

RPWord: mov     al,[ecx+1]
        call    TxtByte
RPSingle:
        mov     al,[ecx]
        call    TxtByte
RPNext: inc     esi
        pop     ebx
        add     ebx,320*8               ; next line down
        jmp     RPLoop
RPExit: pop     esi ecx ebx eax
        ret
PanelInfo:                              ; table of display information
        dd      ZRegPC                  ; CPU Registers. Address,Text
        db      'PC',2                  ; and byte count
        dd      ZRegAF
        db      'AF',2
        dd      ZRegBC
        db      'BC',2
        dd      ZRegDE
        db      'DE',2
        dd      ZRegHL
        db      'HL',2
        dd      ZRegSP
        db      'SP',2
        dd      ZRegIff
        db      'IFF',1
        dd      Break1                  ; Breakpoints
        db      'Break 1',3
        dd      Break2
        db      'Break 2',3
        dd      Break3
        db      'Break 3',3
        dd      Break4
        db      'Break 4',3
        dd      Memory+0FF05h           ; display registers
        db      'Tmr Ctr',1
        dd      Memory+0FF06h
        db      'Tmr Mod',1
        dd      Memory+0FF07h
        db      'Tmr Ctl',1
        dd      Memory+0FF0Fh
        db      'IntFlag',1
        dd      Memory+0FF40h
        db      'LCDCont',1
        dd      Memory+0FF41h
        db      'LCDStat',1
        dd      Memory+0FF42h
        db      'ScrollX',1
        dd      Memory+0FF43h
        db      'ScrollY',1
        dd      Memory+0FF44h
        db      'LCD Y',1
        dd      Memory+0FF45h
        db      'YComp',1
        dd      Memory+0FFFFh
        db      'IntEn',1
        dd      0                       ; end of table

EndP RefreshPanel

; ***********************************************************************
; *                                                                     *
; *                          Output a byte                              *
; *                                                                     *
; ***********************************************************************

TxtWord Proc
        push    eax
        mov     al,ah
        call    TxtByte
        pop     eax
        call    TxtByte
        ret
endp TxtWord

TxtByte Proc
        push    eax eax
        shr     eax,4
        call    TxtNibble
        pop     eax
        call    TxtNibble
        pop     eax
        ret
TxtNibble:
        and     al,15
        add     al,48
        cmp     al,58
        jb      NotLetter
        add     al,7
NotLetter:
        call    TxtCharacter
        ret
EndP TxtByte
end
