; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *                     Global Include File                             *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************

include cpu.inc

Ext_GB  equ     ('g')+('b'*256)
Ext_CFG equ     ('G')+('9'*256)+('7'*65536)
Ext_CSC equ     ('C')+('9'*256)+('7'*65536)
Ext_SAV equ     ('s')+('a'*256)+('v'*65536)

Global Debug:Proc
Global LoadImage:Proc                   ; Load ROM Image into memory
Global LoadFile:Proc
Global SaveFile:Proc
Global ReadPCTimer:Proc
Global DisplayLine:Proc                 ; Do one display line.
Global DisplayFrame:Proc                ; Do one display frame
Global RenderLine:Proc                  ; Draw one display line.
Global Initialise:Proc                  ; set everything up
Global Terminate:Proc                   ; close everything down & quit
Global IsEscape:Proc                    ; Exit ?
Global LoadPaletteData:Proc             ; convert byte palette to EAXECXEDX
Global RenderByte:Proc                  ; Render 8 pixels in BX to EBP
Global CopyLogicalPhysical:Proc         ; small screen repaint
Global CopyLogicalLarge:Proc            ; large screen repaint
Global CopyLogicalTweak:Proc            ; large screen repaint (tweaked)
Global UpdateKeyboardByte:Proc          ; update the keyboard byte
Global UpdateJoystickByte:Proc          ; update the joystick byte
Global UpdateFF00:Proc                  ; update FF00 memory location
Global Reset:Proc                       ; Reset the gameboy
Global SerialReset:Proc                 ; Reset serial I/O stuff
Global SerialWrite:Proc                 ; extension of WritePageFF
Global SerialInterrupt:Proc             ; serial interrupt test
Global DrawSprites:Proc                 ; Draw sprites (per frame basis)
Global MapRomPage:Proc                  ; Map ROM onto 2nd 16k of ROM space
Global MapRamPage:Proc                  ; Map RAM Page
Global ResetMappingData:Proc            ; reset mapping masks etc.
Global SaveBatteryRam:Proc              ; Save RAM
Global RaiseInterrupt:Proc              ; raise Interrupt
Global SoundInterruptRoutine:Proc       ; sound interrupt routine
Global DrawSpritesBehind:Proc           ; Draw sprites behind background
Global SoundInitialise:Proc             ; initialise the sound
Global SoundWrite:Proc                  ; Write to a sound register
Global SoundOff:Proc                    ; turn all the sound off
Global CheckAutofireToggle:Proc         ; check for toggling autofire
Global SetVGAPalette:Proc
Global ResetData:Proc
Global ResetMapping:Proc
Global ClearMemory:Proc
Global ToggleDisplayMode:Proc
Global CheckLoadSave:Proc
Global LoadBitmap:Proc
Global DrawBitmap:Proc
Global GetKeyPressed:Proc

Global TxtString:Proc                   ; write string
Global TxtCharacter:Proc                ; write character
Global TxtColour:Proc                   ; set colour
Global TxtEOL:Proc                      ; clear line
Global ClearScreen:Proc
Global SSDraw:Proc
Global SSExtractLine:Proc
Global SSBuildTable:Proc
Global GoDOS:Near
Global FileNameExt:Proc
Global LoadColourScheme:Proc
Global FixCartridge:Proc
Global SwitchDisplayMode:Proc

; ==========================================================================

Global VideoRam:DWord                   ; offset to VGA RAM from DS
Global DisplayImage:Near                ; 64k drawing space
Global ROMImage:Near                    ; space for Loading ROM
Global CartridgeRAM:Near                ; 32k for Cartridge RAM
Global WindowTable:DWord                ; offset for address and data
Global WindowOffset:Byte                ; for window - NORMALLY 8800 and 80
                                        ; but some games like 8000 and 00
Global AutoOnA:Byte
Global AutoOnB:Byte

Global ReversePixel:Near                ; reverse byte lookup table
Global TimerInterrupt:Byte              ; Timer interrupt flag
Global PaintVector:DWord                ; Screen Redrawer
Global DisplayCounter:Byte              ; Display render skip counter
Global KeyboardByte:Byte                ; Current Key Status
Global FastSprites:Byte                 ; fast sprites on ?

Global GBBackground:DWord               ; Background colour scheme
Global GBWindow:DWord                   ; Window colour scheme
Global GBSprite:DWord                   ; Sprite Colour schemes
Global SpriteScan:Byte                  ; Sprites in scanline flag
Global Mode00Cycles:Word                ; cycles in mode 00 normally 205
Global Mode10Cycles:Word                ; cycles in mode 10 normally 84
Global Mode11Cycles:Word                ; cycles in mode 11 normally 168
Global NoInt60:Byte                     ; disable int60
Global Background:Byte                  ; background colour
Global DisplayRate:Byte                 ; Display refresh rate
Global KeyRight:DWord                   ; Right key (start of table of 8)
Global SoundOn:Byte                     ; Sound on byte
Global FMPort:Word                      ; Adlib port (normally $388)
Global SyncOn:Byte                      ; Synchronisation Flag
Global FrameRate:Word                   ; Frames per tick
Global BigScreen:Byte                   ; Big Screen flag
Global ExtraRate:Byte                   ; extra one
Global FileName:DWord                   ; Filename pointer
Global KeyTable:Near                    ; Table of key press (in CS)
Global TermText:Near                    ; Byebye message
Global UserDataStart:Near               ; Start of config data area
Global NoMLInterrupt:Byte               ; Flag to stop multiple Ints
Global AutoFireA:Byte                   ; autofire rates
Global AutoFireB:Byte
Global AutoCtrlA:DWord                  ; autofire on/off keys
Global AutoCtrlB:DWord
Global EnableCheat1:Byte                ; gg cheats on & cheats
Global Cheat1:DWord
Global EnableCheat2:Byte
Global Cheat2:DWord
Global EnableCheat3:Byte
Global Cheat3:DWord
Global EnableCheat4:Byte
Global Cheat4:DWord
Global JoystickPort:Word                ; joystick port address
Global DebugKey:DWord
Global VGBScheme:DWord                  ; VGB Colour scheme array
Global HorzLineCount:Byte
Global BMStart:Word
Global BMEnd:Word
Global BMFrame:Word
Global Hack:Byte
Global SIOBuf:Byte
Global SIOFlag:Byte
Global SIOCount:Byte
Global ROMMask:Byte
Global CurrROMPage:Byte
Global CurrRAMPage:Byte
Global OldPI0to3:Byte
Global NewFF00Value:Byte
Global ResetKey:DWord
Global DisplayKey:DWord
Global PauseKey:DWord
Global SaveGameStatus:Proc
Global LoadGameStatus:Proc
Global Bitmap:Near
Global CheckPaint:Proc
Global KeyPaint:DWord
Global VBIWhenOff:Byte
Global Mode3Vol:Byte
Global OrInt:Byte
Global UseFixes:Byte
Global Fixed:Byte
Global ResetMouse:Proc
Global AccessGUI:Proc
Global GUIDrawObject:Proc
Global NoBgr:Byte
Global EarlyVBI:Byte
Global KillMouse:Byte
