/************************************************************************/
/*                                                                      */
/*					Code generation (reads and writes)					*/
/*                                                                      */
/************************************************************************/

#include "cpu.h"

/************************************************************************/
/*                                                                      */
/*					Atomic read of translated address					*/
/*                                                                      */
/************************************************************************/

void AtomRead(char *Reg86)
{
ASSERT(IS8086REG(Reg86));
printf("    mov %s,[esi]\n",Reg86);		/* Address set up by translate() */
}

/************************************************************************/
/*                                                                      */
/*						Translate address in BX							*/
/*                                                                      */
/************************************************************************/

void AtomTranslate(void)
{
printf("    movzx esi,bl\n"); 		/* ESI = low byte of address */
printf("    movzx edx,bh\n");       /* Mapping table index in edx */
printf("    add   esi,[dword ptr Mapping+edx*4]\n");
}

/************************************************************************/
/*                                                                      */
/*					Atomic write of translated address					*/
/*                                                                      */
/************************************************************************/

void AtomWrite(char *Reg86)
{
int n = ClaimLabel(2);
ASSERT(IS8086REG(Reg86));
printf("    cmp bh,255\n");				/* If < $FFxx (signed) just write it */
printf("    jl short %s\n",SysLabel(n));
printf("    mov cl,%s\n",Reg86);        /* Data to write in CL */
printf("    call WritePageFF\n");		/* Call function (handles I/O,Mapping) */
printf("    jmp short %s\n",			/* Does its own writing (or not) */
						SysLabel(n+1));
Label(SysLabel(n));
printf("    mov [esi],%s\n",Reg86);		/* Normal code,just save register */
Label(SysLabel(n+1));
}

/************************************************************************/
/*                                                                      */
/*					Fetch byte to given register						*/
/*                                                                      */
/************************************************************************/

void Fetch(char *Reg86)
{
ASSERT(IS8086REG(Reg86));
printf("    mov bx,[%s]\n",				/* BX = Program counter & inc */
					GetZRegName("PC"));
printf("    inc [%s]\n",GetZRegName("PC"));
MemRead(Reg86);							/* Read byte at BX */
}

/************************************************************************/
/*                                                                      */
/*					Vector jump on the lookup table						*/
/*                                                                      */
/************************************************************************/

void LookJump(char *Table)
{
printf("    mov bx,[%s]\n",				/* Get PC to BX and inc it */
					GetZRegName("PC"));
printf("    inc [%s]\n",GetZRegName("PC"));
AtomTranslate();						/* Translate into ESI */
printf("    mov dl,[esi]\n");			/* Read the instruction */
printf("    jmp [dword ptr %s+edx*4]\n",Table);

/* NOTE: This ASSUMES AtomTranslate() will clear EDX (which it does !)
   to save on a few cycles. */

}

/************************************************************************/
/*                                                                      */
/*					Fetch a word to bx (destroys cx)					*/
/*                                                                      */
/************************************************************************/

void FetchWord(void)
{
#ifdef QUICK
	printf("    mov bx,[%s]\n",			/* Get PC to BX and inc it */
					GetZRegName("PC"));
	printf("    add [%s],2\n",GetZRegName("PC"));
	AtomTranslate();					/* Translate into ESI */
	printf("    mov bx,[esi]\n");
#else
	Fetch("cl");						/* Get the two parts & put them */
	Fetch("ch");						/* together. */
	printf("    mov bx,cx\n");
#endif
}

/************************************************************************/
/*                                                                      */
/*						Add constant to cycle counter					*/
/*                                                                      */
/************************************************************************/

void AddCycles(int Count)
{
ASSERT(Count > 0 && Count < 50);
printf("    add [%s],%d\n",GetZRegName("cycles"),Count);
}

/************************************************************************/
/*                                                                      */
/*					Generate code to produce data						*/
/*                                                                      */
/************************************************************************/

void Data(char *Name,int Size)
{
ASSERT(ISSTRING(Name));
ASSERT(Size > 0 && Size <= 2);
printf("%s %s 0\n",Name,(Size == 1) ? "db" : "dw");
}

/************************************************************************/
/*                                                                      */
/*						Read from bx to Register						*/
/*                                                                      */
/************************************************************************/

void MemRead(char *Reg86)
{
AtomTranslate();
AtomRead(Reg86);
}

/************************************************************************/
/*                                                                      */
/*						Write from register to bx						*/
/*                                                                      */
/************************************************************************/

void MemWrite(char *Reg86)
{
AtomTranslate();
AtomWrite(Reg86);
}

/************************************************************************/
/*                                                                      */
/*						Clear & Set flags in AH							*/
/*                                                                      */
/************************************************************************/

void ClearFlag(int Mask)
{
printf("    and ah,0%xh\n",255-Mask);
}

void SetFlag(int Mask)
{
printf("    or ah,0%xh\n",Mask);
}

/************************************************************************/
/*                                                                      */
/*						Generate mapping table							*/
/*                                                                      */
/************************************************************************/

void MappingTable(void)
{
unsigned int i,t;
printf(".data\nalign dword\n");
Label("Mapping");
for (i = 0;i < 256;i++)
	{
	t = i;
	if (t >= 0xC0 && t < 0xFE)
				t = (t & 0x1F)+0xC0;
	printf("    dd Memory+256*0%02xh\n",t);
	}
printf(".code\n");
}