; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *                GUI Code - Mouse, Cursor and Entrance                *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************

.386p 
.model flat
.code 
include Gb.Inc

.data
MouseVRAM       dd      0                       ; mouse VRAM Position
MouseBackVRAM   dd      0                       ; background to save to
MouseBack       dd      16 dup (0)              ; saved background
MouseInstalled  dw      0                       ; $FFFF if mouse installed
MouseX          dw      160                     ; Mouse Coordinates
MouseY          dw      100
MouseMoved      db      0                       ; set to 1 if mouse moves
MouseButton     db      0                       ; set to 1 if button pressed
MouseOld        dw      0                       ; old mouse button status
.code

; ***********************************************************************
; *                                                                     *
; *                          Reset the Mouse                            *
; *                                                                     *
; ***********************************************************************

ResetMouse Proc
        cmp     [KillMouse],0
        jnz     RMExit
        push    eax ebx
        mov     ax,0
        int     033h
        mov     [MouseInstalled],ax
        pop     ebx eax
RMExit: ret
EndP ResetMouse

; ***********************************************************************
; *                                                                     *
; *                Redraw the background under the cursor               *
; *                                                                     *
; ***********************************************************************

DrawMouseBackground Proc
        push    eax ebx ecx edx
        cmp     [MouseBackVRAM],0
        jz      DMBExit
        mov     ebx,[MouseBackVRAM]
        mov     ecx,offset MouseBack
        mov     dl,8
DMB1:   mov     eax,[ecx]
        mov     [ebx],eax
        mov     eax,[ecx+4]
        mov     [ebx+4],eax
        add     ecx,8
        add     ebx,320
        dec     dl
        jnz     DMB1
DMBExit:mov     [MouseBackVRAM],0
        pop     edx ecx ebx eax
        ret
EndP DrawMouseBackground

; ***********************************************************************
; *                                                                     *
; *                Draw a cursor (8 x 8) at the mouse posn              *
; *                                                                     *
; ***********************************************************************

DrawMouseCursor Proc
        push    eax ebx ecx edx
        call    DrawMouseBackground
        mov     ebx,[MouseVRAM]
        mov     [MouseBackVRAM],ebx
        mov     ecx,0
        mov     edx,offset MouseBack
DMC1:   mov     eax,[ebx]                       ; save background
        mov     [edx],eax
        mov     eax,[ebx+4]
        mov     [edx+4],eax
        add     edx,8
        push    ebx                             ; work out colour & size
        mov     ah,cl
        mov     al,15
        cmp     ecx,7
        jne     DMC2
        mov     al,0
DMC2:   mov     [ebx],al                        ; draw the body
        inc     ebx
        dec     ah
        jns     DMC2
        pop     ebx
        mov     al,0                            ; draw the border
        mov     [ebx],al
        mov     [ebx+ecx],al
        add     ebx,320
        inc     ecx
        cmp     ecx,8
        jne     DMC1
        pop     edx ecx ebx eax
        ret
EndP DrawMouseCursor

; ***********************************************************************
; *                                                                     *
; *                Update mouse position / button status                *
; *                                                                     *
; ***********************************************************************

UpdateMousePosition Proc
        push    eax ebx ecx edx
        cmp     [MouseInstalled],0FFFFh         ; check mouse is installed
        jne     UMPExit

        mov     ax,3                            ; read mouse button status
        int     033h
        cmp     [MouseOld],bx
        je      NoButtonChange
        mov     [MouseOld],bx
        and     bx,7                            ; just look at the relev bits
        jz      NoButtonChange
        mov     [MouseButton],1                 ; if gone nonzero button down
NoButtonChange:

        mov     ax,0Bh                          ; read mouse movement
        int     033h
        mov     ax,cx                           ; has the mouse moved ?
        add     ax,dx
        jz      UMPExit                         ; if not exit
        mov     [MouseMoved],1                  ; set the moved flag
        add     [MouseX],cx                     ; update the x & y coordinates
        add     [MouseY],dx

        cmp     [MouseX],0                      ; force mouse position in range
        jge     UMP1
        mov     [MouseX],0
UMP1:   cmp     [MouseX],320-8
        jb      UMP2
        mov     [MouseX],320-8
UMP2:   cmp     [MouseY],0
        jge     UMP3
        mov     [MouseY],0
UMP3:   cmp     [MouseY],200-8
        jb      UMP4
        mov     [MouseY],200-8
UMP4:
        xor     eax,eax                         ; calculate vram position
        xor     ebx,ebx
        mov     bx,[MouseY]
        mov     ax,bx
        shl     ebx,2
        add     ebx,eax
        shl     ebx,6
        mov     ax,[MouseX]
        add     ebx,eax
        add     ebx,[VideoRAM]
        mov     [MouseVRAM],ebx

UMPExit:pop     edx ecx ebx eax
        ret
EndP UpdateMousePosition

; ***********************************************************************
; *                                                                     *
; *     Get Click or Key. Returns AX=X,BX=Y if click,or AX=-1,BX=Key    *
; *                                                                     *
; ***********************************************************************

GetClickOrKey Proc
        push    ecx edx

        mov     [MouseMoved],1                  ; force cursor repaint
        mov     [MouseButton],0                 ; no button pressed

GC1:    call    UpdateMousePosition             ; update mouse info
        cmp     [MouseMoved],0                  ; if moved redraw cursor
        jz      GC2
        call    DrawMouseCursor
        mov     [MouseMoved],0
GC2:    cmp     [MouseButton],0                 ; wait for mouse button
        jz      GC1
        call    DrawMouseBackground             ; repaint the background
        mov     ax,[MouseX]
        mov     bx,[MouseY]
        pop     edx ecx
        ret
EndP GetClickOrKey

; ***********************************************************************
; *                                                                     *
; *                   Hook into Main Program for GUI                    *
; *                                                                     *
; ***********************************************************************

AccessGUI Proc
        cmp     [KillMouse],0                   ; mouse not installed
        jnz     AGExit
        call    UpdateMousePosition             ; wait for mouse click
        cmp     [MouseButton],0                 ; when it happens,start the
        jz      AGExit                          ; GUI

        call    GUIDrawObject
        call    GetClickOrKey

        call    ClearScreen                     ; tidy up
        call    [PaintVector]
        mov     [MouseButton],0                 ; GUI finished.
AGExit: ret
EndP AccessGUI

End
