; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *                     Handle hardware stuff                           *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************

.386p 
.model flat 
.code 
include Gb.Inc

.data
Int9Selector    dw      0               ; old keyboard handler selector
Int9Offset      dd      0               ; old keyboard handler offset
DataSel         dw      0
JoystickEnabled db      0               ; is the joystick enabled ?
JoystickByte    db      0               ; joystick byte
JoyX            dw      0               ; x joystick value
JoyY            dw      0               ; y joystick value
XLow            dw      0               ; calibration values
XHigh           dw      0
YLow            dw      0
YHigh           dw      0
StickCount      db      0
.code

; ***********************************************************************
; *                                                                     *
; *                       Initialise everything                         *
; *                                                                     *
; ***********************************************************************

Initialise Proc
        call    ClearMemory                     ; clear main memory
        xor     eax,eax
        mov     edi,offset CartridgeRAM         ; and cartridge RAM
        mov     bx,32768/4
Erase2: mov     [edi],eax
        add     edi,4
        dec     bx
        jnz     Erase2
        mov     ax,32767                        ; very irregular timer interrupts
        call    SetTimerRate
        call    ResetMouse
        call    LoadImage                       ; load the ROM Image
        push    ds                              ; bx = descriptor for data seg
        pop     ebx                             ; (its a 32 bit push !)
        mov     ax,6                            ; DPMI function 6 gets the
        int     31h                             ; base address for a segment.

        shl     ecx,16                          ; into CX:DX - copy this into
        mov     cx,dx                           ; ECX
        mov     ebx,0A0000h                     ; if we subtract it from A0000
        sub     ebx,ecx                         ; we get, in EBX, the offset
        mov     [VideoRAM],ebx                  ; save the video ram offset
        call    SetPalette                      ; set the palette
        call    ClearScreen                     ; clear the screen
        call    LoadColourScheme                ; set up the VGB ColourScheme
        call    SwitchDisplayMode
        mov     ebx,320*192                     ; display information
        call    TxtEOL
        mov     al,15
        call    TxtColour
        mov     edx,offset LoadedTxt
        call    TxtString
        mov     edx,[FileName]                  ; skip filename prelims
        mov     ecx,edx
Search0:mov     al,[edx]
        inc     edx
        cmp     al,'\'
        jne     Search1
        mov     ecx,edx
Search1:cmp     al,0
        jne     Search0
        mov     edx,ecx
        call    TxtString
        mov     edx,offset LoadTxt2
        call    TxtString
        cmp     [Fixed],0
        jz      NoFix
        mov     al,14
        call    TxtColour
        mov     edx,offset FixedTxt
        mov     ebx,320*192+(320-8*3)
        call    TxtString

NoFix:  mov     ax,0204h                        ; get the old Int9 vector
        mov     bl,9                            ; and save it.
        int     31h
        mov     [Int9Selector],cx
        mov     [Int9Offset],edx
        mov     [DataSel],ds                    ; save the data selector.
        mov     ax,0205h                        ; install the new routine
        mov     bl,9
        mov     cx,cs                           ; current selector
        mov     edx,offset NewInt9
        int     031h

        call    SoundInitialise                 ; initialise the sound
                                                ; decide on screen type
NotBigScreen:
        ret
LoadedTxt db    'Loaded "',0
LoadTxt2  db    '" ('
        include buildx.inc
        db      ')',0
FixedTxt  db    'Fix',0
EndP Initialise

; ***********************************************************************
; *                                                                     *
; *                         Clear memory                                *
; *                                                                     *
; ***********************************************************************

ClearMemory Proc
        push    eax ebx edi
        mov     edi,offset Memory               ; clear Memory
        xor     eax,eax
        mov     ebx,32768                       ; from $8000 on
Erase:  mov     [edi],eax
        add     edi,4
        add     bx,4
        jnz     Erase
        pop     edi ebx eax
        ret
EndP ClearMemory

; ***********************************************************************
; *                                                                     *
; *                         Clear the screen                            *
; *                                                                     *
; ***********************************************************************

ClearScreen Proc
        mov     ax,(320*200/4)                  ; clear the background
        mov     cl,[Background]
        mov     ch,cl
        mov     dx,cx
        shl     ecx,16
        mov     cx,dx
        mov     ebx,[VideoRam]
@@Loop1:
        mov     [ebx],ecx
        add     ebx,4
        dec     ax
        jnz     @@loop1
        cmp     [BigScreen],0
        jne     @@Skip1
        call    GBDrawBackground
        call    DrawBitmap
@@Skip1:
        ret
EndP ClearScreen

; ***********************************************************************
; *                                                                     *
; *              Terminate everything and go to DOS                     *
; *                                                                     *
; ***********************************************************************

Terminate Proc
        call    SoundOff                        ; turn the sound off.

        test    [byte ptr KeyTable+03dh],128
        jz      NoKeyDump
        call    KeyDump                         ; useful for scan codes.
NoKeyDump:
     
        mov     ax,0205h                        ; restore the keyboard vector
        mov     bl,9
        mov     cx,[Int9Selector]
        mov     edx,[Int9Offset]
        cmp     edx,0
        jz      NoReset
        int     031h
NoReset:
        call    SaveBatteryRAM                  ; save Memory
        ret

EndP Terminate

; ***********************************************************************
; *                                                                     *
; *                       Set the Timer Frequency                       *
; *                                                                     *
; ***********************************************************************

TimerFrequency Proc
        push    eax eax
        cli
        mov     al,036h
        out     43h,al
        pop     eax
        out     40h,al
        mov     al,ah
        out     40h,al
        pop     eax
        sti
        ret
EndP TimerFrequency
; ***********************************************************************
; *                                                                     *
; *                       Set the VGA Palette                           *
; *                                                                     *
; ***********************************************************************

SetPalette Proc
        mov     cl,16                   ; background palette 16..20
        mov     eax,[GBBackground]
        call    SetPaletteBlock
        mov     cl,20                   ; set window palette 20..23
        mov     eax,[GBWindow]
        call    SetPaletteBlock
        mov     cl,24                   ; set sprite palette 24..31
        mov     eax,[GBSprite]
        call    SetPaletteBlock
        mov     cl,28
        mov     eax,[GBSprite]
        call    SetPaletteBlock
        ret
EndP SetPalette

SetPaletteBlock Proc
        mov     edx,offset @@Mask       ; set a block of 4 colours
        mov     ch,4                    ; with the various shades
@@Loop: push    eax                     ; by anding with a base colour
        and     eax,[edx]
        call    SetVGAPalette
        add     edx,4
        pop     eax
        inc     cl
        dec     ch
        jnz     @@Loop
        ret
@@Mask: dd      00000000h,60606060h,0B0b0b0b0h,0FFFFFFFFh
EndP SetPaletteBlock

; ***********************************************************************
; *                                                                     *
; *       Set the VGA Palette item CL to be EAX (RGB 0..FF)             *
; *                                                                     *
; ***********************************************************************

.data?
SavedVGAPalette dd 256 dup(?)
.code
      
SetVGAPalette Proc
        push    eax ebx ecx edx
        and     ecx,255
        mov     [SavedVGAPalette+ecx*4],eax
        mov     ebx,eax
        mov     al,255
        mov     dx,03c6h
        out     dx,al
        add     dx,2
        mov     al,cl
        out     dx,al
        inc     dx
        mov     eax,ebx
        shr     eax,16
        shr     al,2
        out     dx,al
        mov     eax,ebx
        shr     eax,8
        shr     al,2
        out     dx,al
        mov     eax,ebx
        shr     al,2
        out     dx,al
        pop     edx ecx ebx eax
        ret
EndP SetVGAPalette

; ***********************************************************************
; *                                                                     *
; *                          New Int 9 handler                          *
; *                                                                     *
; ***********************************************************************

NewInt9:push    eax ebx
        push    ds
        mov     ds,cs:[DataSel]

        in      al,60h                  ; get the scan code

        mov     bl,al                   ; bx = offset into the table
        and     ebx,07Fh

        and     al,080h                 ; isolate the on off bit
        xor     al,080h                 ; bit 7 0 if key pressed,reverse it
        mov     [byte ptr KeyTable+ebx],al ; write it into the table

        mov     al,20h                  ; reset something or other....
        out     20h,al

        pop     ds
        pop     ebx eax
        sti
        iret

.data
KeyTable:
         db     128 dup (0)             ; keyboard table IN THE CODE SEGMENT
.code

; ***********************************************************************
; *                                                                     *
; *                         New Int9 Handler                            *
; *                                                                     *
; ***********************************************************************

KeyDump Proc
@@Loop7:test    [byte ptr KeyTable+1],128
        jnz     @@Loop7
@@Loop6:mov     eax,0
@@Loop4:test    [byte ptr KeyTable+eax],128
        jz      @@Loop5
        cmp     eax,02ah
        je      @@Loop5
        call    DebugByte
        push    eax
        mov     al,32
        call    DebugChar
        call    DebugChar
        pop     eax
@@Loop5:inc     eax
        cmp     eax,128
        jne     @@Loop4
        test    [byte ptr KeyTable+1],128
        jz      @@Loop6
        ret
EndP KeyDump

; ***********************************************************************
; *                                                                     *
; *                     Check for escape pressed                        *
; *                                                                     *
; ***********************************************************************

IsEscape Proc
        mov     al,[byte ptr KeyTable+1]
        and     al,128
        ret
EndP IsEscape

; ***********************************************************************
; *                                                                     *
; *                     Update the keyboard byte                        *
; *                                                                     *
; ***********************************************************************

UpdateKeyboardByte Proc
        push    eax ebx ecx
        mov     ebx,offset KeyRight     ; bx ^ key table numbers
        mov     al,1                    ; al bit pattern
        xor     ah,ah                   ; ah result
@@UKB:  mov     ecx,[ebx]               ; get the offset
        test    [Byte Ptr KeyTable+ecx],128
        jz      @@UKB2
        or      ah,al                   ; set bit if pressed
@@UKB2: add     ebx,4                   ; next in table
        add     al,al                   ; shift bit left
        jnz     @@UKB
        or      ah,[JoystickByte]       ; or with joystick byte
        mov     [KeyboardByte],ah       ; save result

        mov     al,[AutoFireA]          ; check autofire a
        and     eax,0FFh                ; make it 32 bit
        jz      NoAutoA1
        cmp     [AutoOnA],0
        jz      NoAutoA1
        mov     ax,[word ptr AutoFireRate+eax*2]
        and     ax,[BMFrame]            ; fire rate mask and frame count
        jz      NoAutoA1
        and     [KeyboardByte],255-16
NoAutoA1:
        mov     al,[AutoFireB]          ; check autofire b
        and     eax,0FFh                ; make it 32 bit
        jz      NoAutoB1
        cmp     [AutoOnB],0
        jz      NoAutoB1
        mov     ax,[word ptr AutoFireRate+eax*2]
        and     ax,[BMFrame]            ; fire rate mask and frame count
        jz      NoAutoB1
        and     [KeyboardByte],255-32
NoAutoB1:
        pop     ecx ebx eax
        ret

AutoFireRate:                           ; autofire rate table
        dw      0,1,2,4,8,16,32,64,128,256

EndP UpdateKeyboardByte

; ***********************************************************************
; *                                                                     *
; *    If scan code in EAX pressed,toggle autofire pointed to by EBX    *
; *                                                                     *
; ***********************************************************************

CheckAutofireToggle Proc
        test    [byte ptr KeyTable+eax],128     ; is key pressed
        jz      CATExit
CATWait:test    [byte ptr KeyTable+eax],128     ; wait for release
        jnz     CATWait
        mov     [CATType],cl
        mov     al,1                            ; toggle it.
        mov     [CATSwitch],'N'
        mov     [CATSwitch+1],0
        xor     [ebx],al
        jnz     CATSwitchOn
        mov     [CATSwitch],'F'
        mov     [CATSwitch+1],'F'
        mov     [CATSwitch+2],0
CATSwitchOn:
        mov     al,14                           ; set the colour
        call    TxtColour
        mov     ebx,320*192                     ; write the message
        call    TxtEOL
        mov     edx,offset CATText
        call    TxtString
CATExit:
        ret
.data
CATText db      'Autofire '                     ; displayable message
CATType db      'x o'
CATSwitch db    'xx',0
.code
EndP CheckAutofireToggle

; ***********************************************************************
; *                                                                     *
; *                    Update the Joystick Byte                         *
; *                                                                     *
; ***********************************************************************

UpdateJoystickByte Proc
        cmp     [JoystickEnabled],0             ; if joystick is enabled
        jnz     JoystickIsInUse                 ; read it.
        mov     dx,[JoystickPort]               ; read the buttons
        cmp     dx,0                            ; if port is zero then
        jz      JoystickExit                    ; joystick is disabled
        in      al,dx
        and     al,0F0h                         ; check if either button
        cmp     al,0F0h                         ; pressed - if so enable
        je      JoystickExit                    ; the joystick

        mov     [JoystickEnabled],1             ; enable it
        call    ReadJoystickDirect
        mov     ax,[JoyX]
        call    DoCalibration                   ; convert to calibration values
        mov     [XLow],ax
        mov     [XHigh],bx
        mov     ax,[JoyY]
        call    DoCalibration                   ; convert to calibration values
        mov     [YLow],ax
        mov     [YHigh],bx

        mov     ebx,320*192                     ; display information
        call    TxtEOL
        mov     al,11
        call    TxtColour
        mov     edx,offset StickOn
        call    TxtString
        jmp     JoystickExit                    ; and exit.
StickOn db      'Joystick Enabled',0

DoCalibration:
        mov     cx,ax                           ; bx = cx = ax
        mov     bx,ax
        shr     cx,1                            ; cx = centre / 2
        sub     ax,cx                           ; ax = centre - centre/2
        add     bx,cx                           ; bx = centre + centre/2
        ret

JoystickIsInUse:
        mov     dx,[JoystickPort]               ; read the buttons
        in      al,dx
        and     al,0F0h                         ; get the buttons
        xor     al,0F0h                         ; make so on = 1
        mov     [JoystickByte],al               ; that represents A & B

        inc     [StickCount]
        mov     al,[StickCount]
        and     al,3
        jnz     NoRead
        call    ReadJoystickDirect
NoRead:

        mov     ax,[JoyX]
        cmp     ax,[XLow]
        jae     Skip1
        or      [JoystickByte],2
Skip1:  cmp     ax,[XHigh]
        jb      Skip2
        or      [JoystickByte],1
Skip2:  mov     ax,[JoyY]                       ; now do the same for Y
        cmp     ax,[YLow]
        jae     Skip3
        or      [JoystickByte],4
Skip3:  cmp     ax,[YHigh]
        jb      Skip4
        or      [JoystickByte],8
Skip4:

JoystickExit:
        ret
EndP UpdateJoystickByte

; ***********************************************************************
; *                                                                     *
; *                   Read Joystick into JoyX JoyY                      *
; *                                                                     *
; ***********************************************************************

ReadJoystickDirect Proc

        push    eax ebx ecx edx                 ; read using BIOS
        mov     ah,84h
        mov     dx,1
        int     15h
        mov     [JoyX],ax
        mov     [JoyY],bx
        pop     edx ecx ebx eax
        ret

        mov     al,1                            ; read X axis
        call    ReadJoystick
        mov     [JoyX],ax
        mov     al,2                            ; read Y axis
        call    ReadJoystick
        mov     [JoyY],ax
        ret
EndP ReadJoystickDirect

ReadJoystick Proc
        push    ebx ecx edx
        cli                                     ; interrupts off
        mov     dx,[JoystickPort]               ; get the joystick port
        xor     cx,cx                           ; clear the count
        out     dx,al                           ; write a dummy byte to start
        out     dx,al
        mov     ah,al                           ; put the mask in ah
ReadJoy:inc     cx
        jz      ReadJoyEnd                      ; overflow ?
        in      al,dx                           ; look at the port
        and     al,ah                           ; is the bit set ?
        jnz     ReadJoy                         ; wait for it to go to zero
ReadJoyEnd:
        mov     ax,cx                           ; return value in AX
        sti                                     ; interrupts on again
        pop     edx ecx ebx
        ret
EndP ReadJoystick

; ***********************************************************************
; *                                                                     *
; *                       Draw the background                           *
; *                                                                     *
; ***********************************************************************

GBDrawBackground Proc
        mov     ecx,offset BackgroundGfx
GBDB1:  mov     ax,[word ptr ecx+4]             ; ax = line number.
        cmp     ax,-1
        je      GBDBExit
        mov     bx,ax                           ; ax = ax * 320
        shl     bx,2
        add     ax,bx                           ; ax = (ax * 5)
        shl     ax,6                            ; ax = (ax * 5 * 64)
        add     ax,[word ptr ecx+0]             ; start of line
        movzx   eax,ax                          ; extend it to 32 bits
        add     eax,[VideoRAM]                  ; points to videoram
        push    eax
        mov     bx,[word ptr ecx+6]             ; bx = draw colour
        mov     dx,[word ptr ecx+2]             ; dx = width
GBDB2:  mov     [eax],bl
        inc     eax
        dec     dx
        jnz     GBDB2
        mov     bx,[word ptr ecx+8]             ; draw the beginning point
        mov     [eax],bl
        pop     eax
        mov     [eax],bl
        add     ecx,10                          ; go to the next line
        jmp     GBDB1

GBDBExit:
        ret
BackgroundGfx:                                  ; table of x1,x2,y,line,end
        include Gfx.Inc
        dw      -1,-1,-1,-1,-1                  ; end of table marker
EndP GBDrawBackground

; ***********************************************************************
; *                                                                     *
; *            Set up for the translated ColourSchemes                  *
; *                                                                     *
; ***********************************************************************

LoadColourScheme Proc
        mov     eax,[dword ptr VGBScheme]       ; colour scheme loaded ?
        add     eax,[dword ptr VGBScheme+4]
        add     eax,[dword ptr VGBScheme+8]
        add     eax,[dword ptr VGBScheme+12]
        jz      LCSExit                         ; if not,exit
        mov     ebx,offset VGBScheme
        mov     cl,10h
LCSLoop:mov     eax,[ebx]
        call    SetVGAPalette
        add     ebx,4
        inc     cl
        cmp     cl,20h
        jnz     LCSLoop

LCSExit:ret
EndP LoadColourScheme

; ***********************************************************************
; *                                                                     *
; *                Check for display mode toggling                      *
; *                                                                     *
; ***********************************************************************

ToggleDisplayMode Proc
        push    eax ebx
        mov     eax,[DisplayKey]
        mov     bl,[byte ptr KeyTable+eax]
        test    bl,128
        jz      TDMExit
TDMWait:mov     bl,[byte ptr KeyTable+eax]
        test    bl,128
        jnz     TDMWait
        inc     [BigScreen]
        cmp     [BigScreen],3
        jb      TDMNoReset
        mov     [BigScreen],0
TDMNoReset:
        call    SwitchDisplayMode
TDMExit:mov     eax,[PauseKey]
        mov     bl,[byte ptr KeyTable+eax]
        test    bl,128
        jz      TPAExit
        mov     ebx,320*192                     ; display information
        call    TxtEOL
        mov     al,14
        call    TxtColour
        mov     edx,offset Paused
        call    TxtString
        call    WaitPause
        call    WaitPause
        mov     ebx,320*192                     ; display information
        call    TxtEOL
TPAExit:pop     ebx eax
        ret
WaitPause:
        mov     eax,[PauseKey]
WP1:    mov     bl,[byte ptr KeyTable+eax]
        test    bl,128
        jz      WP1
WP2:    mov     bl,[byte ptr KeyTable+eax]
        test    bl,128
        jnz     WP2
        ret
Paused: db      'Game Paused.',0
EndP ToggleDisplayMode


; ***********************************************************************
; *                                                                     *
; *                         Set display mode                            *
; *                                                                     *
; ***********************************************************************

SwitchDisplayMode Proc
        mov     ah,0                    ; switch to 320x200 VGA mode
        mov     al,013h
        int     010h
        mov     ecx,16                  ; restore the palette
TDMPal: mov     eax,[SavedVGAPalette+ecx*4]
        call    SetVGAPalette
        inc     cl
        jnz     TDMPal

        cmp     [BigScreen],2
        je      ToTweak
        cmp     [BigScreen],1           ; decide on the redraw mode
        je      ToLarge                 ; 0 = Normal,1 = Stretch,2=Tweak
        mov     [PaintVector],offset CopyLogicalPhysical
        jmp     TDMRedraw
ToLarge:mov     [PaintVector],offset CopyLogicalLarge
        jmp     TDMRedraw
ToTweak:call    Set360x144Mode
        mov     [PaintVector],offset CopyLogicalTweak
TDMRedraw:
        call    ClearScreen
        call    [PaintVector]
        ret
EndP SwitchDisplayMode

; ***********************************************************************
; *                                                                     *
; *                       Set 360 x 144 mode                            *
; *                                                                     *
; ***********************************************************************

Set360x144Mode Proc
 mov dx,03c4h
 mov ax,0100h
 out dx,ax
 mov dx,03c2h
 mov al,027h
 out dx,al
 mov dx,03c4h
 mov ax,0300h
 out dx,ax
 mov dx,03d4h
 mov ax,02e11h
 out dx,ax
 mov dx,03d4h
 mov ax,06b00h
 out dx,ax
 mov dx,03d4h
 mov ax,05901h
 out dx,ax
 mov dx,03d4h
 mov ax,05a02h
 out dx,ax
 mov dx,03d4h
 mov ax,08e03h
 out dx,ax
 mov dx,03d4h
 mov ax,05e04h
 out dx,ax
 mov dx,03d4h
 mov ax,08a05h
 out dx,ax
 mov dx,03d4h
 mov ax,05f06h
 out dx,ax
 mov dx,03d4h
 mov ax,01f07h
 out dx,ax
 mov dx,03d4h
 mov ax,08h
 out dx,ax
 mov dx,03d4h
 mov ax,04109h
 out dx,ax
 mov dx,03d4h
 mov ax,0ah
 out dx,ax
 mov dx,03d4h
 mov ax,0bh
 out dx,ax
 mov dx,03d4h
 mov ax,0ch
 out dx,ax
 mov dx,03d4h
 mov ax,0dh
 out dx,ax
 mov dx,03d4h
 mov ax,0eh
 out dx,ax
 mov dx,03d4h
 mov ax,0fh
 out dx,ax
 mov dx,03d4h
 mov ax,03c10h
 out dx,ax
 mov dx,03d4h
 mov ax,02e11h
 out dx,ax
 mov dx,03d4h
 mov ax,02f12h
 out dx,ax
 mov dx,03d4h
 mov ax,02d13h
 out dx,ax
 mov dx,03d4h
 mov ax,04014h
 out dx,ax
 mov dx,03d4h
 mov ax,03615h
 out dx,ax
 mov dx,03d4h
 mov ax,05916h
 out dx,ax
 mov dx,03d4h
 mov ax,0a317h
 out dx,ax
        ret
EndP Set360x144Mode

end

