; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *                     Handles loading of the program                  *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************

.386p 
.model flat 
.code 
include Gb.Inc


; ***********************************************************************
; *                                                                     *
; *                     Load the ROM Image into memory                  *
; *                                                                     *
; ***********************************************************************

LoadImage Proc
        mov     eax,Ext_GB              ; append gb extension
        call    FileNameExt
        lea     esi,ROMImage            ; address to load
        call    LoadFile                ; load the ROM image
        cmp     ax,-1
        je      Failed

        mov     edx,offset DefaultCfg   ; load default cfg file
        mov     esi,offset UserDataStart
        call    LoadFile

        mov     eax,Ext_CFG             ; load <cart>.g97
        call    FileNameExt             ; configuration file for rom
        mov     esi,offset UserDataStart
        call    LoadFile

        mov     eax,Ext_CSC             ; load <cart>.c97
        call    FileNameExt             ; colour scheme for rom
        mov     esi,offset VGBScheme
        call    LoadFile
        call    LoadBitmap              ; load the bitmap

        call    GameGenie               ; do the game genie patches
        call    FixCartridge            ; fix any known carts
        call    HackROM                 ; various patches

        mov     eax,Ext_SAV             ; load sav file
        call    FileNameExt
        lea     esi,CartridgeRAM
        call    LoadFile                ; ignore errors.

        call    Reset                   ; reset the CPU
        call    ResetData               ; reset the BSS data
        call    ResetMapping            ; reset the mapping
        call    ResetMappingData        ; and the masks
        ret

Failed: mov     ah,0                    ; switch to text display mode
        mov     al,03h
        int     010h
        mov     edx,offset LoadFailedTxt
        mov     ah,9
        int     21h
        mov     edx,offset TermText
        mov     ah,9
        int     21h
        mov     ax,4C00h
        int     21h
.data
LoadFailedTxt:                          ; cart couldn't load message
        db      10,'That ROM Image couldn',39,'t be loaded.',13,10,'$'
DefaultCFG:                             ; default configuration file
        db      'Default.G97',0

EndP LoadImage

; ***********************************************************************
; *                                                                     *
; *     Load File. EDX^Filename, ESI^Memory location. Returns -1        *
; *     if failed,or the number of bytes allocated                      *
; *                                                                     *
; ***********************************************************************

LoadFile Proc
        mov     al,0                    ; open the file.
        mov     ah,03Dh                 ; DOS function 3Dh
        int     21h
        jc      LoadFailed              ; oh no,it failed.
        push    ax                      ; save the handle
        mov     bx,ax                   ; put it in AX
        mov     edx,esi                 ; address to load in DX
        mov     ecx,4194304             ; bytes to load - 4mb should be enough
        mov     ah,3Fh                  ; DOS function 3Fh is load
        int     21h
        pop     bx                      ; resetore file handle
        push    eax                     ; save bytes loaded
        mov     ah,3Eh                  ; & close the file using DOS Func 3Eh
        int     21h
        pop     eax                     ; restore the bytes read.
        jmp     ExitLoad
LoadFailed:
        mov     eax,-1                  ; return -1
ExitLoad:
        ret

EndP LoadFile

; ***********************************************************************
; *                                                                     *
; *                       Reset the gameboy                             *
; *                                                                     *
; ***********************************************************************

Reset Proc
        call    CPUReset
        mov     [byte ptr Memory+0FF40h],081h           ; LCD Cont
        mov     [byte ptr Memory+0FF45h],0              ; LCD Compare
        mov     [byte ptr Memory+0FF48h],0E4h           ; palettes
        mov     [byte ptr Memory+0FF49h],0E4h
        mov     [byte ptr Memory+0FF47h],0E4h
        mov     [byte ptr Memory+0FF05h],1              ; timer
        mov     [byte ptr Memory+0FF06h],1
        mov     [byte ptr Memory+0FF07h],0F8h
        call    ResetMapping                            ; reset the mapping
        ret
EndP Reset

; ***********************************************************************
; *                                                                     *
; *              Reset the mapping table for the first 32k              *
; *                                                                     *
; ***********************************************************************

ResetMapping Proc
        mov     eax,offset ROMImage                     ; data pointer
        mov     ebx,offset Mapping                      ; ebx ^ mapping table
        mov     cl,128                                  ; no of maps to write
RMap1:  mov     [ebx],eax                               ; write out one map
        add     eax,256                                 ; next page
        add     ebx,4                                   ; next mapping slot
        dec     cl                                      ; do 128 times
        jnz     RMap1
        mov     al,0                                    ; map initial RAM Page
        call    MapRAMPage
        ret

EndP ResetMapping

; ***********************************************************************
; *                                                                     *
; *      Map the 16k ROM at EAX onto the 2nd 16k of ROM Space in        *
; *     the gameboy (4000-7FFF)                                         *
; *                                                                     *
; ***********************************************************************

MapROMPage Proc
        include maprom.asm
        ret
EndP MapROMPage

; ***********************************************************************
; *                                                                     *
; *        Get filename with new ext EAX, return pointer in EDX         *
; *                                                                     *
; ***********************************************************************

FileNameExt Proc
        push    esi edi
        mov     edi,[FileName]                          ; edi = org filename
        mov     esi,offset FNBuffer                     ; esi = new filename
FNELoop:mov     dl,[edi]                                ; copy until . or \0
        mov     [esi],dl                                ; found.
        inc     esi
        inc     edi
        cmp     dl,'.'
        je      FNEDone
        cmp     dl,32
        ja      FNELoop
FNEDone:mov     [esi],eax                               ; add new extension
        mov     dl,'.'                                  ; add the .
        mov     [esi-1],dl
        mov     edx,offset FNBuffer                     ; edx points to buffer
        pop     edi esi
        ret
.data
FNBuffer db     127 dup (0)                             ; buffer
.code
EndP FileNameExt

; ***********************************************************************
; *                                                                     *
; *                     Save Battery RAM to Disk                        *
; *                                                                     *
; ***********************************************************************

SaveBatteryRAM Proc
        mov     al,[byte ptr ROMImage+0147h]            ; check type 3 or 6
        cmp     al,3                                    ; (3=MBC1+Battery)
        je      SBR1                                    ; (6=MBC2+Battery)
        cmp     al,6
        jne     SBRExit
SBR1:   mov     eax,Ext_SAV                             ; edx = filename
        call    FileNameExt
        mov     ecx,8192                                ; ecx = size
        mov     al,[byte ptr ROMImage+0149h]            ; check if 32k RAM
        cmp     al,3
        jne     SBR2
        mov     ecx,32768
SBR2:   mov     eax,offset CartridgeRAM                 ; eax = start address
        call    SaveFile                                ; Save the file

SBRExit:ret
EndP SaveBatteryRAM

; ***********************************************************************
; *                                                                     *
; *          Save file name EDX from memory at EAX size ECX             *
; *                                                                     *
; ***********************************************************************

SaveFile Proc
        push    eax ebx ecx edx                         ; save everything
        push    eax ecx                                 ; save start & size
        push    edx                                     ; save filename
        mov     ah,41h                                  ; delete file
        int     21h
        pop     edx                                     ; edx = filename
        xor     cx,cx                                   ; cx = attribute
        mov     ah,3Ch
        int     21h                                     ; create the file
        pop     ecx                                     ; restore size
        pop     edx                                     ; restore address
        push    eax                                     ; save handle
        mov     bx,ax                                   ; put in bx
        mov     ah,40h                                  ; and write
        int     21h
        pop     ebx                                     ; restore handle
        mov     ah,3Eh                                  ; close file
        int     21h
        pop     edx ecx ebx eax
        ret

EndP SaveFile

; ***********************************************************************
; *                                                                     *
; *            Patch the Game Genie Cheats in if enabled                *
; *                                                                     *
; ***********************************************************************

GameGenie Proc
        push    eax ebx
        mov     al,[EnableCheat1]                       ; patch the cheats
        mov     ebx,[Cheat1]
        call    PatchGenie
        mov     al,[EnableCheat2]
        mov     ebx,[Cheat2]
        call    PatchGenie
        mov     al,[EnableCheat3]
        mov     ebx,[Cheat3]
        call    PatchGenie
        mov     al,[EnableCheat4]
        mov     ebx,[Cheat4]
        call    PatchGenie
        pop     ebx eax
        ret
EndP GameGenie

; ***********************************************************************
; *                                                                     *
; *       Possibly patch the game genie cheat in AL=Flag EBX=Patch      *
; *                                                                     *
; ***********************************************************************

PatchGenie Proc
        push    eax ebx ecx edx
        cmp     al,0                    ; don't do it if the patch flag isnt set
        jz      PGExit

        mov     eax,ebx                 ; dl = replace value
        shr     eax,24
        mov     dl,al

        mov     eax,ebx                 ; ecx = offset
        shr     eax,8                   ; eax now contains the address
        ror     ax,4
        xor     ax,0F000h               ; complement the MS Nibble
        movzx   ecx,ax

        mov     al,bl                   ; dh = replace value
        xor     al,0FFh
        ror     al,2
        xor     al,045h
        mov     dh,al

        cmp     ecx,2000h               ; in first ROM bank
        jae     PGInROMBank             ; if not search the banks available

        mov     [byte ptr ROMImage+ecx],dl
        jmp     PGExit

PGInROMBank:
        add     ecx,offset ROMImage     ; points to first bank hackable.
        mov     bl,[byte ptr ROMImage+0149h]
        movzx   ebx,bl                  ; ebx is ROM Number
        mov     bl,[BankCount+ebx]      ; number of banks to check.
PGIRLoop:
        cmp     [ecx],dh                ; does it match
        jne     NoMatch

        mov     [ecx],dl                ; if so patch it.
NoMatch:add     ecx,16384               ; point ecx to next bank
        dec     bl
        jnz     PGIRLoop                ; and do for each bank

PGExit: pop     edx ecx ebx eax
        ret
BankCount db    1,3,7,15,31,63,127
EndP PatchGenie

; ***********************************************************************
; *                                                                     *
; *                         Hack various games                          *
; *                                                                     *
; ***********************************************************************

HackROM Proc
        mov     al,[Hack]               ; which hack
        cmp     al,1
        je      HHawk
        ret

HHawk:  mov     al,255                 ; hack hudson hawk & kill screens
        mov     [byte ptr ROMImage+03922h],al
        ret

EndP HackROM

end
