; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *                           Sound Code                                *
; *                                                                     *
; *               this is version 2, all code by PSR                    *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************
.386p
.model flat
.code
include gb.inc

.data
SoundReg        db 64 dup (0)           ; current sound registers value
.code
        
; ***********************************************************************
; *                                                                     *
; *                     Initialise Soundcard                            *
; *                                                                     *
; ***********************************************************************
                           
SoundInitialise Proc
        mov     ah,0                    ; start up sound
        call    Sound
        mov     [byte ptr Memory+0FF26h],08Fh
        mov     [byte ptr SoundReg+026h],08Fh
        ret
EndP SoundInitialise

; ***********************************************************************
; *                                                                     *
; *                        Write to soundcard                           *
; *                                                                     *
; *   On entry CL = Data,ESI = Physical Address,BX = Logical Address    *
; *                                                                     *
; ***********************************************************************

SoundWrite Proc
        push    eax ebx ecx edx esi edi
        cmp     bx,0FF10h                       ; check range >= $FF10
        jb      SWExit
        cmp     bx,0FF26h                       ; check range < $FF26
        ja      SWExit
        movzx   ebx,bl                          ; ebx = 32 bit of low address
        mov     [byte ptr SoundReg+ebx],cl      ; save the result.
        sub     ebx,010h                        ; now ebx is $00 to $$16
        call    [dword ptr SoundHandlers+ebx*4] ; call a routine
SWExit: pop     edi esi edx ecx ebx eax
Noop:   ret
SoundHandlers:
        dd      Noop                            ; $FF10 set sweep 1
        dd      Noop                            ; $FF11 set len/patt 1
        dd      WriteFF12                       ; $FF12 set sound volume 1
        dd      WriteFF13                       ; $FF13 set frequency low 1
        dd      WriteFF14                       ; $FF14 set frequency high 1
        dd      Noop                            ; $FF15 (unknown)
        dd      Noop                            ; $FF16 set len/patt 2
        dd      WriteFF17                       ; $FF17 set sound volume 2
        dd      WriteFF18                       ; $FF18 set frequency low 2
        dd      WriteFF19                       ; $FF19 set frequency high 2
        dd      WriteFF1A                       ; $FF1A set on/off 3
        dd      Noop                            ; $FF1B set sound length 3
        dd      WriteFF1C                       ; $FF1C set typ  3
        dd      WriteFF1D                       ; $FF1D set frequency low 3
        dd      WriteFF1E                       ; $FF1E set frequency high 3
        dd      Noop                            ; $FF1F (unknown)
        dd      Noop                            ; $FF20 set sound length 4
        dd      WriteFF21                       ; $FF21 set sound volume 4
        dd      WriteFF22                       ; $FF22 set mode 4 polynomial
        dd      Noop                            ; $FF23 set mode 4 count/cons
        dd      WriteFF24                       ; $FF24 set stereo out on/off
        dd      WriteFF25                       ; $FF25 set stereo channels
        dd      WriteFF26                       ; $FF26 global mute

; ***********************************************************************
; ***                          Main Control                           ***
; ***********************************************************************

WriteFF26:
WriteFF24:
WriteFF25:
        mov     al,[byte ptr SoundReg+025h]     ; get sound output selections
        mov     ah,[byte ptr SoundReg+024h]     ; get stereo controls
        test    ah,128                          ; if bit 7 set then ch2 is on
        jnz     SO2On
;       and     al,0Fh                          ; no. turn off all so2 chan
SO2on:  test    ah,16                           ; if bit 7 set then ch1 is on
        jnz     SO1On
;       and     al,0F0h                         ; no. turn off all so1 chan
SO1on:  mov     ah,al                           ; put the 2 channels together
        shr     ah,4                            ; so that we know which is on
        or      al,ah                           ; either on either channel
        and     al,15                           ; means the channel is on.
        mov     cl,al                           ; this is now in cl

        mov     al,[byte ptr SoundReg+026h]     ; are any channels on ?

        and     al,128
        jz      GlobalCtrl
        or      al,15                           ; turn channel marker on.
GlobalCtrl:
        mov     [byte ptr Memory+0FF26h],al     ; update the value
        mov     [byte ptr SoundReg+026h],al
        and     cl,al                           ; and with channel controls
        mov     al,0                            ; work through the channels
TurnOnOff:
        mov     ah,8                            ; channel mute on
        test    cl,1
        jz      TurnOff
        mov     ah,9                            ; channel mute off
TurnOff:call    Sound                           ; do it for that channel
        shr     cl,1                            ; do it for all 4
        inc     al
        cmp     al,4
        jne     TurnOnOff

        ret

; ***********************************************************************
; ***                       MODE 1 CHANNEL 0                          ***
; ***********************************************************************

WriteFF12:                                      ; $FF12 Envelope Mode 1
        shr     cl,4                            ; high bits are volume
        and     cl,15
        mov     ah,2                            ; 2 = set volume
        mov     al,0                            ; channel 0
        mov     bl,cl                           ; bl = volume
        call    Sound
        ret

WriteFF14:                                      ; $FF13 Frequency Mode 1 (Hi)
        mov     al,255-64                       ; only bit 6 is readable
        and     [esi],al                        ; in $FF13
        test    cl,128                          ; reset sound envelope ?
        jz      WriteFF13
        mov     cl,[SoundReg+012h]
        call    WriteFF12
WriteFF13:                                      ; $FF14 Frequency Mode 1 (Lo)
        mov     bx,[word ptr SoundReg+013h]     ; get the frequency
        and     bx,07FFh
        mov     ah,4                            ; 4 = set frequency
        mov     al,0                            ; channel 0
        call    Sound
        ret

; ***********************************************************************
; ***                       MODE 2 CHANNEL 1                          ***
; ***********************************************************************

WriteFF17:                                      ; $FF17 Envelope Mode 2
        shr     cl,4                            ; high bits are volume
        and     cl,15
        mov     ah,2                            ; 2 = set volume
        mov     al,1                            ; channel 1
        mov     bl,cl                           ; bl = volume
        call    Sound
        ret

WriteFF19:                                      ; $FF19 Frequency Mode 2 (Hi)
        mov     al,255-64                       ; only bit 6 is readable
        and     [esi],al                        ; in $FF19
        test    cl,128                          ; reset sound envelope ?
        jz      WriteFF18
        mov     cl,[SoundReg+017h]
        call    WriteFF17
WriteFF18:                                      ; $FF18 Frequency Mode 2 (Lo)
        mov     bx,[word ptr SoundReg+018h]     ; get the frequency
        and     bx,07FFh
        mov     ah,4                            ; 4 = set frequency
        mov     al,1                            ; channel 1
        call    Sound
        ret

; ***********************************************************************
; ***                       MODE 3 CHANNEL 2                          ***
; ***********************************************************************

WriteFF1A:                                      ; $FF1A Channel on/off
WriteFF1C:                                      ; $FF1C Level
        mov     bl,0                            ; volume
        mov     al,[byte ptr SoundReg+01Ah]
        test    al,128                          ; must be turned on
        jz      Write3Cont
        mov     al,[byte ptr SoundReg+01Ch]     ; and level must be nonzero
        and     al,030h
        jz      Write3Cont
        mov     bl,[Mode3Vol]
Write3Cont:                                     ; set the volume of channel 3
        mov     ah,2
        mov     al,2
        call    Sound
        ret

WriteFF1E:                                      ; $FF1E Frequency Mode 1 (Hi)
        mov     al,255-64                       ; only bit 6 is readable
        and     [esi],al                        ; in $FF1E
WriteFF1D:                                      ; $FF1D Frequency Mode 1 (Lo)
        mov     bx,[word ptr SoundReg+01Dh]     ; get the frequency
        and     bx,07FFh
        mov     ah,4                            ; 4 = set frequency
        mov     al,2                            ; channel 2
        call    Sound
        ret

; ***********************************************************************
; ***                       MODE 4 CHANNEL 3                          ***
; ***********************************************************************

WriteFF21:                                      ; $FF21 Envelope Mode 4
        shr     cl,4                            ; high bits are volume
        and     cl,15
        mov     ah,2                            ; 2 = set volume
        mov     al,3                            ; channel 3
        mov     bl,cl                           ; bl = volume
        call    Sound
        ret

WriteFF22:
        mov     al,3                            ; channel 3
        mov     ah,4                            ; set frequency
        mov     bx,1100                         ; to 1100
        call    Sound
        ret

EndP SoundWrite

; ***********************************************************************
; *                                                                     *
; *                        Turn off all sound                           *
; *                                                                     *
; ***********************************************************************

SoundOff Proc
        mov     ah,1                    ; terminate sound
        call    Sound
EndP SoundOff

; ***********************************************************************
; *                                                                     *
; *                     Sound Interrupt Routine                         *
; *                                                                     *
; ***********************************************************************

SoundInterruptRoutine Proc
        push    eax ebx ecx edx
        call    FrequencySweep
        call    FrequencySweep
        call    FrequencySweep
        call    Length4
        call    Length4
        call    Length4
        call    Length4
        call    EnvelopeSweep
        pop     edx ecx ebx eax
        ret

Length4:mov     al,[byte ptr SoundReg+20h]
        cmp     al,0
        jz      L4Exit
        dec     [byte ptr SoundReg+20h]
        jnz     L4Exit
        mov     ah,2
        mov     al,3
        mov     bl,0
        call    Sound
L4Exit: ret

EnvelopeSweep:
        mov     eax,0                   ; channel 0
        mov     edx,12h
        call    DoEnvelopeSweep
        mov     eax,1
        mov     edx,17h
        call    DoEnvelopeSweep
        mov     eax,3
        mov     edx,21h
        call    DoEnvelopeSweep
        ret

FrequencySweep:
        mov     eax,0                   ; channel number 0
        mov     edx,010h                ; pointer to registers
        call    DoFrequencySweep
        ret

EndP SoundInterruptRoutine

ESCounters db   0,0,0,0                         ; counters for envelope sweep

DoEnvelopeSweep Proc
        dec     [byte ptr ESCounters+eax]       ; decrement counter
        js      DES1                            ; if negative, sweep
        jnz     DESExit                         ; exit if non-zero
DES1:   mov     bl,[byte ptr SoundReg+edx]      ; get the sweep register
        and     ebx,7                           ; bottom 3 bits only
        jz      DESExit                         ; if zero,exit (no sweep)
        add     bl,bl
        mov     [byte ptr ESCounters+eax],bl    ; reset the counter

        mov     ah,3                            ; get volume into bl
        call    Sound

        mov     ah,[byte ptr SoundReg+edx]      ; check direction
        and     ah,8
        jz      DESDown
        inc     bl                              ; increment envelope
        cmp     bl,15
        jbe     DESWrite
        mov     bl,15
        je      DESWrite
DESDown:dec     bl                              ; decrement envelope
        jns     DESWrite
        xor     bl,bl
DESWrite:mov    ah,2                            ; rewrite volume
        call    Sound
DESExit:ret

EndP DoEnvelopeSweep
; ***********************************************************************
; *                                                                     *
; *     Do a Frequency Sweep on Channel EAX, EDX is register offset     *
; *                                                                     *
; ***********************************************************************

DoFrequencySweep Proc
        xor     ebx,ebx                         ; clear ebx
        mov     bl,[byte ptr SoundReg+edx]      ; look at the control byte
        mov     cl,[byte ptr FSTab+ebx]         ; get the frequency sweep
        movsx   ecx,cl                          ; sign extend to 32 bits
        mov     ah,5
        call    Sound                           ; get frequency
        add     bx,cx                           ; change the value
        cmp     bx,0                            ; check the range
        jge     DFS1
        xor     bx,bx
DFS1:   cmp     bx,2048
        jb      DFS2
        mov     bx,2047
DFS2:   mov     ah,4                            ; rewrite the frequency
        call    Sound
        ret
EndP DoFrequencySweep

; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *                  Medium Level Sound Interface                       *
; *                                                                     *
; * Sound (AH = Function,AL=Channel,BX = Data (BL if 8 bit))            *
; *     0 = Startup                                                     *
; *     1 = Terminate                                                   *
; *     2 = SetVolume                                                   *
; *     3 = GetVolume                                                   *
; *     4 = SetFrequency                                                *
; *     5 = GetFrequency                                                *
; *     6 = Global Mute On                                              *
; *     7 = Global Mute Off                                             *
; *     8 = Channel Mute On                                             *
; *     9 = Channel Mute Off                                            *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************

Sound   Proc
        push    eax ecx edx


;       push    eax
;       call    DebugWord
;       mov     al,32
;       call    DebugChar
;       mov     ax,bx
;       call    DebugWord
;       mov     al,13
;       call    DebugChar
;       mov     al,10
;       call    DebugChar
;       pop     eax

        movzx   ecx,ah                          ; ecx = 32 bit function
        movzx   eax,al                          ; eax = 32 bit channel
        call    [dword ptr SoundVect+ecx*4]     ; call the function
        pop     edx ecx eax
        ret
SoundVect:
        dd      SMInit                          ; function 0 = Start
        dd      SMMuteOn                        ; function 1 = End
        dd      SMSetVol                        ; function 2 = Set Vol
        dd      SMGetVol                        ; function 3 = Get Vol
        dd      SMSetFreq                       ; function 4 = Set Frq
        dd      SMGetFreq                       ; function 5 = Get Frq
        dd      SMMuteOn                        ; function 6 = Mute on
        dd      SMMuteOff                       ; function 7 = Mute off
        dd      SMCMuteOn                       ; function 8 = ChMute on
        dd      SMCMuteOff                      ; function 9 = ChMute off

SMInit: call    DetectAdlib
        xor     ax,ax
        xor     bx,bx
SMI0:   call    SetFrequency
        call    SetVolume
        inc     al
        cmp     al,4
        jne     SMI0
        ret
SMSetVol:
        cmp     bl,[byte ptr CVolume+eax]       ; volume changed ?
        je      SMReturn                        ; if not do nothing
        mov     [byte ptr CVolume+eax],bl       ; update volume
ResetVolume:
        mov     cl,[byte ptr CMute+eax]         ; cl = Mute
        mov     ah,[byte ptr CVolume+eax]       ; ah = volume
        and     ah,[Mute]                       ; and with global mute status
        and     ah,cl                           ; and with channel mute status
        call    SetVolume                       ; call the primitive
SMReturn:
        ret
SMGetVol:                                       ; return the volume
        mov     bl,[byte ptr CVolume+eax]
        xor     bh,bh
        ret
SMSetFreq:
        cmp     bx,[word ptr CFreq+eax*2]       ; frequency changed
        je      SMReturn                        ; if not do nothing
        mov     [word ptr CFreq+eax*2],bx       ; update frequency entry
        call    SetFrequency                    ; call the primitive
        ret
SMGetFreq:
        mov     bx,[word ptr CFreq+eax*2]       ; get current frequency
        ret
SMMuteOn:                                       ; mute all channels
        mov     [Mute],0                        ; clear mute
        xor     ax,ax                           ; ah = al = 0
SMMO2:  call    ResetVolume                     ; all channels off
        xor     ah,ah
        inc     al
        cmp     al,4
        jne     SMMO2
        ret
SMMuteOff:
        mov     [Mute],255
        xor     eax,eax                         ; ax = 0
SMMO1:  mov     ah,[byte ptr CVolume+eax]       ; restore the volume
        call    SetVolume
        xor     ah,ah
        inc     al
        cmp     al,4
        jnz     SMMO1
        ret
SMCMuteOn:                                      ; mute channel
        mov     [byte ptr CMute+eax],ah         ; turn the mute off (ah=0)
        call    SetVolume                       ; ch al volume = ah
        ret
SMCMuteOff:
        mov     cl,255                          ; demute the channel
        cmp     [byte ptr CMute+eax],cl         ; if already on ignore it
        je      SMCMO1
        mov     [byte ptr CMute+eax],cl
        call    ResetVolume
SMCMO1: ret

.data
Mute    db      255                             ; mute mask. 255=off,0=on
align word
CVolume db      0,0,0,0                         ; current volume (chn 0..3)
CFreq   dw      0,0,0,0                         ; current freq (chn 0..3)
CMute   db      255,255,255,255                 ; current mutes (chn 0..3)
.code
EndP Sound

; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *               Low level interface to Adlib card                     *
; *                                                                     *
; * DetectAdlibCard()            Check existence of card & set it up    *
; * SetVolume(AL=Channel,AH=Vol) Set Channel AL to volume BL (0..15)    *
; * SetFrequency(AL=Chan,BX=Frq) Set Channel AL to Freq BX (0..2047)    *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************

.data
AdlibFound      db      0                       ; 1 if card found,0 otherwise

align dword                                     ; get the tables
        include sounddat.inc
.code

; ***********************************************************************
; *                                                                     *
; *                 Write AL to ADLIB Register AH                       *
; *                                                                     *
; ***********************************************************************

WriteFM Proc
        push    eax ebx ecx edx
        mov     dx,[FMPort]             ; dx points to adlib card
        xchg    al,ah
        out     dx,al                   ; output the address
        mov     ecx,256                 ; wait a bit
WFM1:   dec     ecx
        jnz     WFM1
        xchg    al,ah                   ; output the data
        inc     dx
        out     dx,al
        mov     ecx,768                 ; wait a bit again
WFM2:   dec     ecx
        jnz     WFM2
        pop     edx ecx ebx eax
        ret
EndP WriteFM

; ***********************************************************************
; *                                                                     *
; *                 Read Adlib Status Byte to AL                        *
; *                                                                     *
; ***********************************************************************

ReadFM  Proc
        push    edx
        mov     dx,[FMPort]
        in      al,dx
        pop     edx
        ret
EndP ReadFM

; ***********************************************************************
; *                                                                     *
; *                 Detect & Initialise Adlib card                      *
; *                                                                     *
; ***********************************************************************

DetectAdlib Proc
        push    eax ebx ecx edx
        mov     [AdlibFound],0                  ; not found
        mov     ax,0100h                        ; writefm(1,0)
        call    WriteFM                         ; disable test register
        mov     ax,0460h                        ; writefm(4,60)
        call    WriteFM                         ; disable timer/stop
        mov     ax,0480h                        ; writefm(4,80)
        call    WriteFM                         ; delete status register
        call    ReadFM                          ; read the card
        and     al,0E0h                         ; x & 0xE0 must be zero
        jnz     DetectExit
        mov     ax,02FFh                        ; writefm(2,ff)
        call    WriteFM                         ; counter 1 = 255
        mov     ax,0421h                        ; writefm(4,21)
        call    WriteFM                         ; connection on/start timer
        mov     cx,200                          ; wait a bit
DAD1:   call    ReadFM                          ; read the port 200 times
        dec     cx
        jnz     DAD1
        and     al,0E0h                         ; must be 0xC0
        cmp     al,0C0h
        jne     DetectExit
        mov     [AdlibFound],1                  ; we've found it
        mov     ax,0460h                        ; writefm(4,60)
        call    WriteFM                         ; disable timer/stop
        mov     ax,0480h                        ; writefm(4,80)
        call    WriteFM                         ; delete status register
        mov     ax,0800h                        ; writefm(8,0)
        call    WriteFM                         ; melody mode on
        mov     ax,0BD00h                       ; writefm(BD,0)
        call    WriteFM                         ; vibrato off

        mov     ebx,offset AdlibSetupData       ; initialise the card
SIN1:   mov     ah,[ebx+0]                      ; send out a stream of
        mov     al,[ebx+1]                      ; data to the card setting
        call    WriteFM                         ; up the channels
        add     ebx,2
        mov     al,[ebx+0]                      ; terminated by zero
        cmp     al,0
        jnz     SIN1
DetectExit:
        pop     edx ecx ebx eax
        ret
EndP DetectAdlib

; ***********************************************************************
; *                                                                     *
; *               Set Frequency of Channel AL to BX                     *
; *                                                                     *
; ***********************************************************************

SetFrequency Proc
        push    eax ebx
        cmp     [AdlibFound],0                  ; check card
        jz      SFQExit
        cmp     bx,2048                         ; must be 0..2047
        jae     SFQExit
        mov     ah,al                           ; ah = Channel+0A0h
        add     ah,0A0h
        movzx   ebx,bx                          ; extend bx to 32 bits
        mov     al,[byte ptr FreqData+ebx*2]    ; read the frequency (low)
        call    WriteFM                         ; WriteFM(Channel+0xA0,low)
        add     ah,010h                         ; ah = Channel+0B0h
        mov     al,[byte ptr FreqData+ebx*2+1]  ; read the frequency (high)
        call    WriteFM
SFQExit:pop     ebx eax
        ret
EndP SetFrequency

; ***********************************************************************
; *                                                                     *
; *                  Set Volume of Channel AL to AH                     *
; *                                                                     *
; ***********************************************************************

SetVolume Proc
        push    eax ebx ecx
        movzx   ebx,ah                          ; ebx = 32 bit volume
        cmp     ebx,16                          ; must be in range 0..15
        jae     SVLExit
        mov     cl,[byte ptr VolData+ebx]       ; cl = volume value
        mov     bl,al                           ; ebx = 32 bit channel
        mov     ah,[byte ptr InstAddr+ebx]      ; ah = register to write
        add     ah,043h                         ; = instad[]+0x43
        mov     al,63                           ; al = 63 - volume
        sub     al,cl
        call    WriteFM                         ; write the volume
SVLExit:pop     ecx ebx eax
        ret
EndP SetVolume
    
end
