; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *                        Sprite drawing code                          *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************

.386p 
.model flat 
.code 
include Gb.Inc

; ***********************************************************************
; *                                                                     *
; *          Draw sprites on virtual screen starting at EBP             *
; *                                                                     *
; ***********************************************************************

DrawSprites Proc
        mov     esi,0FE00h
DrawSpr1:
        call    DrawOneSprite
        add     esi,4
        cmp     esi,0FEA0h
        jne     DrawSpr1
        ret
EndP DrawSprites

; ***********************************************************************
; *                                                                     *
; *          Draw sprites on virtual screen starting at EBP             *
; *                                                                     *
; ***********************************************************************

DrawOneSprite Proc
        push    edi esi ebp
; ==========================================================================
;          get x coordinate into eax, y coordinate into ebx,check range
; ==========================================================================
        movzx   eax,[byte ptr Memory+esi+1]     ; al = x coordinate
        sub     eax,8                           ; adjust it
        movzx   ebx,[byte ptr Memory+esi]       ; bl = y coordinate
        sub     ebx,16                          ; adjust it
        cmp     eax,-8                          ; eax must be -7..159
        jle     DrawOSExit
        cmp     eax,160
        jge     DrawOSExit
        cmp     ebx,-16                         ; ebx must be -15..143
        jle     DrawOSExit
        cmp     ebx,144
        jge     DrawOSExit
; ==========================================================================
;           work out address of sprite as x + y * 256 into ebp
; ==========================================================================
        shl     ebx,8                           ; multiply bx by 256
        add     ebp,eax                         ; add x
        add     ebp,ebx                         ; add y * 256
; ==========================================================================
;                    work out address of tile into edi
; ==========================================================================
        movzx   edi,[byte ptr Memory+esi+2]     ; edi = tile number
        test    [byte ptr Memory+0FF40h],4      ; using 8 x 16 sprites ?
        jz      Not8x16
        and     edi,0FEh                        ; 8x16 sprites always even
Not8x16:shl     edi,4                           ; times 16 (16 bytes per patt)
        add     edi,offset Memory+8000h         ; point to pattern data.
; ==========================================================================
;                  load the appropriate palette
; ==========================================================================
        test    [byte ptr Memory+esi+3],16      ; bit 4 of offset 3
        mov     bl,[byte ptr Memory+0FF48h]     ; object palette 0
        jz      SpriteLoadPalette0
        mov     bl,[byte ptr Memory+0FF49h]     ; object palette 1
SpriteLoadPalette0:
        call    LoadPaletteData                 ; load the palette data
        add     eax,08080808h                   ; adjust for sprite palette
        add     ecx,08080808h                   ; which is colours 18..1B
        add     edx,08080808h
        test    [byte ptr Memory+esi+3],16      ; if object palette 1
        jz      SpriteModifyPalette0
        add     eax,04040404h                   ; adjust for sprite palette
        add     ecx,04040404h                   ; which is colours 1C..1F
        add     edx,04040404h

SpriteModifyPalette0:
        mov     [SprLoopCtr],8                  ; no of pixels to write
        test    [byte ptr Memory+0FF40h],4      ; if 8x16 sprites
        jz      Not8x16b
        mov     [SprLoopCtr],16                 ; write 16 pixels
Not8X16b:
        mov     [AdjustEDI],2                   ; direction.
; ==========================================================================
;  if its upside down, add height * 2 - 2 to the tile pointer & reverse
;  the tile ptr increment direction.
; ==========================================================================
        test    [byte ptr Memory+esi+3],64      ; check if upside-down
        jz      NoUpsideDown
        xor     ebx,ebx                         ; ebx = [height]
        mov     bl,[SprLoopCtr]                 ; from the loop counter
        dec     bl                              ; decrement 1
        add     ebx,ebx                         ; double it
        add     edi,ebx                         ; adjust the tile pointer
        mov     [AdjustEDI],-2
NoUpsideDown:
; ==========================================================================
;               Check for x direction & dispatch appropriately
; ==========================================================================
        test    [byte ptr Memory+esi+3],32      ; reverse in x direction ?
        jnz     WriteSpriteReverse
; ==========================================================================
;                       Write sprite normally (forward)
; ==========================================================================
WriteSpriteForward:
        mov     bx,[edi]                        ; get the tile byte
        call    RenderByte                      ; render it
        add     ebp,256-8                       ; next screen position
        add     edi,[AdjustEDI]                 ; next pattern to write
        dec     [SprLoopCtr]                    ; loop round sufficient times
        jnz     WriteSpriteForward
        jmp     DrawOSExit
; ==========================================================================
;                          Write sprite in reverse
; ==========================================================================
WriteSpriteReverse:
        mov     bx,[edi]                        ; get the tile byte
        push    eax                             ; reverse it
        movzx   eax,bl
        mov     bl,[byte ptr ReversePixel+eax]
        movzx   eax,bh
        mov     bh,[byte ptr ReversePixel+eax]
        pop     eax
        call    RenderByte                      ; render it
        add     ebp,256-8                       ; next screen position
        add     edi,[AdjustEDI]                 ; next pattern to write
        dec     [SprLoopCtr]                    ; loop round sufficient times
        jnz     WriteSpriteReverse
        jmp     DrawOSExit

DrawOSExit:
        pop     ebp esi edi
        ret

ReverseBL Proc
EndP ReverseBL

.data
SprLoopCtr      db      0                       ; loop counter for sprites
AdjustEDI       dd      0                       ; direction for pattern change
.code

EndP DrawOneSprite
end
