; ***********************************************************************
; ***********************************************************************
; *                                                                     *
; *                   Handle timing and interrupts                      *
; *                                                                     *
; ***********************************************************************
; ***********************************************************************

.386p 
.model flat
.data
align dword
FrameCount      dw      0
ExtraCount      db      1
.code 
include Gb.Inc

; ***********************************************************************
; *                                                                     *
; *                    Do one frame of the display                      *
; *                                                                     *
; ***********************************************************************

DisplayFrame Proc
        call    ReadPCTimer             ; read & save the timer
        push    ax
        mov     ax,[FrameRate]
        mov     [FrameCount],ax
        dec     [ExtraCount]            ; bung in extra frames every so
        jnz     DisplayFrameLoop        ; often.
        inc     [FrameCount]
        mov     bl,[ExtraRate]
        mov     [ExtraCount],bl
DisplayFrameLoop:
        call    DisplayLine             ; do one display line
        cmp     [byte ptr Memory+0FF44h],70
        jne     NotMiddle
        call    UpdateKeyboardByte      ; scan the keys
        call    CheckSingleFrameStep
NotMiddle:
        cmp     [byte ptr Memory+0FF44h],0
        jnz     DisplayFrameLoop        ; loop round until frame completed

        call    SoundInterruptRoutine   ; update sound
        inc     [DisplayCounter]        ; increment the frame display counter
        mov     al,[DisplayCounter]     ; if reached the frame display rate
        cmp     al,[DisplayRate]        ; reset to zero - the frame will be
        jb      NoResetCounter          ; displayed the next time around
        mov     [DisplayCounter],0
NoResetCounter:
        inc     [BMFrame]
        call    UpdateFF00              ; update the keypad scan register
        dec     [FrameCount]
        jnz     DisplayFrameLoop

        mov     eax,[AutoCtrlA]         ; toggle autofire check
        mov     ebx,offset AutoOnA
        mov     cl,'A'
        call    CheckAutoFireToggle
        mov     eax,[AutoCtrlB]
        mov     ebx,offset AutoOnB
        mov     cl,'B'
        call    CheckAutoFireToggle

        pop     bx                      ; restore the pc timer value
        call    UpdateJoystickByte      ; scan the joystick byte
        mov     al,[SyncOn]             ; check if sync is on
        cmp     al,0
        jz      ExitFrame               ; if not run flat out
        mov     al,[byte ptr KeyTable+04Eh]   ; or if keypad + pressed
        cmp     al,0
        jnz     ExitFrame
WaitTimer:
        call    ReadPCTimer             ; wait for the timer to change
        cmp     ax,bx
        je      WaitTimer
ExitFrame:


        ret
EndP DisplayFrame

CheckSingleFrameStep Proc
        push    eax
        mov     al,[byte ptr KeyTable+01Fh]     ; is S pressed ?
        and     al,128
        jz      ExitSCheck
WaitReleaseS:                           ; wait for release
        mov     al,[byte ptr KeyTable+01Fh]
        and     al,128
        jnz     WaitReleaseS
WaitAnother:                            ; wait for S or Space
        mov     al,[byte ptr KeyTable+01Fh]
        or      al,[byte ptr KeyTable+039h]
        and     al,128
        jz      WaitAnother
ExitSCheck:
        pop     eax
        ret
EndP CheckSingleFrameStep

; ***********************************************************************
; *                                                                     *
; *                    Read the PC's 18.2 Hz Timer                      *
; *                                                                     *
; ***********************************************************************

ReadPCTimer Proc
        push    ebx ecx edx
        mov     ah,0                    ; look at the clock tick
        int     01Ah
        mov     ax,dx                   ; put the tick in AX
        pop     edx ecx ebx
        ret
EndP ReadPCTimer

; ***********************************************************************
; *                                                                     *
; *                 Do a complete line of the display                   *
; *                                                                     *
; *  as : Fetch1 (84 cycles) Fetch2 (168 cycles) HBlank (205 cycles)    *
; *  takes 457 cycles or 108us. 154 of these take 16.6ms or one frame   *
; *                                                                     *
; *  note : Modes 10 and 11 interrupts are not enabled                  *
; *                                                                     *
; ***********************************************************************

DisplayLine Proc
; ============================================================================
;             set the mode to 10 (fetch from OAM) for 84 cycles.
; ============================================================================
        cmp     [byte Ptr Memory+0FF44h],144    ; Only set if not in VBLANK
        jae     InVBlank1
        and     [byte Ptr Memory+0FF41h],0FCh   ; set LCDSTAT bits 0-1 to
        or      [byte Ptr Memory+0FF41h],2      ; 10 = Fetch from OAM mode.
        mov     ax,[Mode10Cycles]               ; do 84 (default) CPU Cycles.
        call    ExecuteCycles
        inc     [byte ptr Memory+0FF04h]        ; increment divreg
; ============================================================================
;                              Display the line..
; ============================================================================
        cmp     [DisplayCounter],0              ; need to render the line ?
        jnz     InVBlank1                       ; only if the displaycounter
        call    RenderLine                      ; is zero
InVBlank1:
; ============================================================================
;       Go to mode 11 (fetch from OAM and Video RAM) and wait for
;       168 cycles.
; ============================================================================
        cmp     [byte Ptr Memory+0FF44h],144    ; Only set if not in VBLANK
        jae     InVBlank2
        and     [byte Ptr Memory+0FF41h],0FCh   ; set LCDSTAT bits 0-1 to
        or      [byte Ptr Memory+0FF41h],3      ; 11 = Fetch from OAM/VRAM mode.
InVBlank2:
        mov     ax,[Mode11Cycles]               ; do 168 (default) CPU Cycles.
        shr     ax,1                            ; in 2 chunks with the
        push    ax                              ; serial interrupt in the
        call    ExecuteCycles                   ; middle
        inc     [byte ptr Memory+0FF04h]        ; increment divreg
        call    SerialInterrupt                 ; check for serial interrupt
        pop     ax
        call    ExecuteCycles                   ; do the other half
        inc     [byte ptr Memory+0FF04h]        ; increment divreg
; ============================================================================
;             Trigger HBLANK interrupt. (an LCDC status interrupt)
; ============================================================================
        test    [byte Ptr Memory+0FF41h],8      ; bit 3 of LCDSTAT must be
        jz      NoHBlankInterrupt               ; set to enable this int.
        call    LCDCInterrupt
NoHBlankInterrupt:
; ============================================================================
;              now trigger the VBlank interrupt, if its required.
; ============================================================================
        cmp     [EarlyVBI],0
        jz      NotVBI1
        call    CheckVBIRequired
NotVBI1:
; ============================================================================
;         Go to mode 00 (Horizontal Blanking) and wait for 205 Cycles
; ============================================================================
        cmp     [byte Ptr Memory+0FF44h],144    ; Only set if not in VBLANK
        jae     InVBlank3
        and     [byte Ptr Memory+0FF41h],0FCh   ; set LCDSTAT bits 0-1 to
        or      [byte Ptr Memory+0FF41h],0      ; 00 = Horizontal Blanking
InVBlank3:
        mov     ax,[Mode00Cycles]               ; do 205 (default) CPU Cycles.
        shr     ax,1                            ; in 2 sections, if the
        push    ax                              ; VGB style timer is wanted
        call    ExecuteCycles
        inc     [byte ptr Memory+0FF04h]        ; increment divreg
        pop     ax                              ; other half of this block
        call    ExecuteCycles
        inc     [byte ptr Memory+0FF04h]        ; increment divreg
; ============================================================================
;       go to the next line. Increment the current line counter and
;       wrap it around 0..153. Set the coincidence flag if the current
;       line = coincidence line. Go to VBLANK if line = 144
; ============================================================================
        inc     [byte Ptr Memory+0FF44h]        ; increment current line
        mov     al,[HorzLineCount]
        cmp     [byte Ptr Memory+0FF44h],al     ; reached end of screen ?
        jb      NotTopOfFrame                   ; if so, wrap around
        mov     [byte Ptr Memory+0FF44h],0      ; back to the start.
NotTopOfFrame:

        and     [byte Ptr Memory+0FF41h],0FBh   ; clear the coincidence flag
        mov     al,[Byte Ptr Memory+0FF44h]     ; if line = coincidence line
        cmp     al,[Byte Ptr Memory+0FF45h]     ; then set the coincidence flag
        jne     NoCoincidence
        or      [byte Ptr Memory+0FF41h],04h    ; set the flag
        test    [byte ptr Memory+0FF41h],040h   ; check if coinc int enabled
        jz      NoCoincidence
        call    LCDCInterrupt                   ; if so generate it.
NoCoincidence:

        cmp     [byte Ptr Memory+0FF44h],144    ; going into VBLANK mode ?
        jne     NotEnteringVBlank
        and     [byte Ptr Memory+0FF41h],0FCh   ; set LCDSTAT bits 0-1 to
        or      [byte Ptr Memory+0FF41h],1      ; 01 = VBLANK mode.
NotEnteringVBlank:
; ============================================================================
;              now trigger the VBlank interrupt, if its required.
; ============================================================================
        cmp     [EarlyVBI],0
        jnz     NotVBI2
        call    CheckVBIRequired
NotVBI2:
        ret
EndP DisplayLine


CheckVBIRequired Proc
        cmp     [byte Ptr Memory+0FF44h],144    ; only on scan line 144
        jne     NoVBlankInterrupt
        cmp     [VBIWhenOff],0                  ; fix for Frank Thornton
        jnz     GenVBI
        test    [byte Ptr Memory+0FF40h],128    ; the screen must be on
        jz      NoVBlankInterrupt
GenVBI: mov     al,01h                          ; raise Interrupt 1
        call    RaiseInterrupt                  ; (vblank)
NoVBlankInterrupt:
        ret
EndP    CheckVBIRequired
; ***********************************************************************
; *                                                                     *
; *         Generates an LCDC Status interrupt if its enabled           *
; *                                                                     *
; ***********************************************************************

LCDCInterrupt Proc
        test    [Byte Ptr Memory+0FF40h],128    ; screen must be on , too
        jz      NoLCDCInt
        push    eax
        mov     al,2                            ; raise interrupt 2
        call    RaiseInterrupt                  ; (LCD Stat)
        pop     eax
NoLCDCInt:
        ret
EndP LCDCInterrupt

; ***********************************************************************
; *                                                                     *
; *             Called by CPU when cycle counter times out.             *
; *                                                                     *
; ***********************************************************************

TimedOut Proc
        test    [byte Ptr Memory+0FF07h],4      ; is the timer running ?
        jz      short TimedOut1                 ; if not, exit now.

        inc     [byte Ptr Memory+0FF05h]        ; Increment Timer Counter
        jnz     short TimedOut1                 ; if not overflowed,exit now.

        push    eax
        mov     al,[byte ptr Memory+0FF06h]     ; reset the timer counter
        mov     [byte ptr Memory+0FF05h],al     ; from timer modulo.

        mov     al,4                            ; raise interrupt 04h
        call    RaiseInterrupt                  ; (timer)
        pop     eax

TimedOut1:
        ret

EndP TimedOut

; ***********************************************************************
; *                                                                     *
; *                        Raise an interrupt                           *
; *                                                                     *
; ***********************************************************************

RaiseInterrupt Proc
        cmp     [NoMLInterrupt],0               ; stops recursive interrupts..
        jz      RaiseAlways                     ; (can't set FF0F in Z80 DI)
        cmp     [zregIff],0                     ; hack to get Bubble Bobble 1
        jz      DontRaise                       ; working.
RaiseAlways:
        test    [byte ptr Memory+0FFFFh],al     ; check interrupt is enabled
        jz      DontRaise
        cmp     [OrInt],0
        jnz     DoOrInt

        mov     [byte ptr Memory+0FF0Fh],al     ; raise it.
        jmp     DontRaise
DoOrInt:or      [byte ptr Memory+0FF0Fh],al     ; raise it by ORing
DontRaise:
        ret
EndP RaiseInterrupt

end

